
#ifndef __DIALOGS_H
#define __DIALOGS_H

#define BUTTON(ID) ((uMsg == WM_COMMAND) && (wParam == LOWORD(ID)))

#ifndef __SETUP_H
#include "SetupFiles.h"
#endif

class WARSETUP_DLL PropertyPageList;

class WARSETUP_DLL PropertyPage
{
public:
	PropertyPage(Setup *pSetup);
	
	HWND m_hWnd;
	BOOL m_IsOK;
	LPCSTR m_Name;
	BOOL m_bEnabled;

	enum MessageTypesDef
	{
		MYWM_ACTIVATE = WM_USER + 10,
		MYWM_NEXT,
		MYWM_PREV,
		MYWM_FINISH,
		MYWM_OK,
		MYWM_UPDATE,
		MYWM_PROGRESS,
		MYWM_PROCESS
	} MessageTypes;

	virtual Setup *GetSetup();
	virtual BOOL Create(int DlgID, HWND WndParent, HINSTANCE hInstance = hInst);
	virtual BOOL OnActivate();
	virtual BOOL OnNext();
	virtual BOOL OnPrev();
	virtual BOOL OnFinish();
	virtual BOOL OnProcess();
	virtual BOOL OnOK();
	virtual BOOL OnInitDlg();
	virtual BOOL OnCreate();
	virtual void OnUpdate(BOOL bSave = TRUE);
	virtual BOOL OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam);
	
	static BOOL CALLBACK PropertyProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
	static PropertyPage *m_pCurrent; // Dirty hack
	PropertyPageList *m_pOrigin;
	Setup *m_pSetup;

protected:
	UINT m_uMessage;
	WPARAM m_wParam;
	LPARAM m_lParam;
};

class WARSETUP_DLL PropertyPageList
{
public:
	PropertyPageList(Setup *pSetup);
	BOOL AddPage(PropertyPage *pPage);
	BOOL ActivatePage(int Page = 0);
	BOOL GoNext();
	BOOL GoPrev();
	BOOL IsFirst();
	BOOL IsLast();
	PropertyPage *GetFirst();
	PropertyPage *GetLast();
	PropertyPage *GetCurr();
	PropertyPage *GetPage(LPCSTR Name);
	BOOL SetAsFirst(LPCSTR Name);
	BOOL IsEnabled(LPCSTR Name);
	BOOL Enable(LPCSTR Name, BOOL bEnable);
	void EnableCancelButton(BOOL bEnable);
	void EnableNextButton(BOOL bEnable);
	void EnablePrevButton(BOOL bEnable);

	enum
	{
		MAX_PAGES = 32
	};

	PropertyPage *m_pDlg[MAX_PAGES];
	int m_NumPages;
	int m_Current;
	int m_First;
	Setup *m_pSetup;
};

class WARSETUP_DLL WelcomePage : public PropertyPage
{
public:
	WelcomePage(Setup *pSetup);
	virtual BOOL OnInitDlg();
	BOOL OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam);
	HICON m_hIcon;
};

class WARSETUP_DLL InstallDirPage : public PropertyPage
{
public:
	InstallDirPage(Setup *pSetup);

	virtual BOOL OnInitDlg();
	virtual BOOL OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam);
	virtual BOOL OnOK();
	virtual void OnUpdate(BOOL bSave = TRUE);
	void OnBrowse();

	char m_Path[MAX_PATH];
};

class WARSETUP_DLL InstallModePage : public PropertyPage
{
public:
	InstallModePage(Setup *pSetup);

	virtual BOOL OnInitDlg();
	virtual void OnUpdate(BOOL bSave = TRUE);
	virtual BOOL OnOK();

	enum InstallModesDef
	{
		mode_normal,
		mode_minimum,
		mode_custom,
		mode_invalid
	} m_Mode;
};


class WARSETUP_DLL InstallCustomPage : public PropertyPage
{
public:
	InstallCustomPage(Setup *pSetup);

	enum 
	{
		NUM_MODULES = 8
	};

	struct ModuleInfo
	{
		BOOL m_bSelected;
		CClassList *m_pClass;
	} m_Data[NUM_MODULES];

	virtual BOOL OnInitDlg();
	virtual void OnUpdate(BOOL bSave = TRUE);
	virtual BOOL OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam);
	void SetState(int Index);
	BOOL GetState(int Index);
};

class WARSETUP_DLL StartMenuPage : public PropertyPage
{
public:
	StartMenuPage(Setup *pSetup);

	virtual BOOL OnInitDlg();
	virtual void OnUpdate(BOOL bSave = TRUE);
	virtual BOOL OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam);

	char m_StartMenu[128];
	BOOL m_bAddStartMenu;
};

class WARSETUP_DLL FileList
{
public:
	FileList(Setup *pSetup, CClassList *pClass);
	void DoModal(HWND hParent);
	int DoModal();
	static BOOL CALLBACK FileListPrc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
	BOOL OnInitdialog(HWND hWnd);

	Setup *m_pSetup; 
	CClassList *m_pClass;
	static FileList *m_pCurrent;
};


class WARSETUP_DLL BeginCopyPage : public PropertyPage
{
public:
	BeginCopyPage(Setup *pSetup);
};

class WARSETUP_DLL CopyFilesPage : public PropertyPage
{
public:
	CopyFilesPage(Setup *pSetup);
	virtual BOOL OnActivate();
	virtual BOOL OnProcess();
};

class WARSETUP_DLL DonePage : public PropertyPage
{
public:
	DonePage(Setup *pSetup);
	virtual BOOL OnActivate();
	virtual BOOL OnInitDlg();
};


#endif // __DIALOGS_H