

#include <windows.h>
#include <string.h>
#include <stdio.h>

#include "WarSetup.h"
#include "resource.h"
#include "Dialogs.h"
#include "SetupFiles.h"

extern void DummyFunc();

// Local prototypes
int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow);
BOOL InitApplication(HINSTANCE hInstance);
BOOL InitInstance(HINSTANCE hInstance,int nCmdShow);
BOOL CALLBACK SetupProc(HWND hwndDlg,UINT uMsg,WPARAM wParam,LPARAM lParam);
LRESULT CALLBACK WndProgressProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);
void LoadPlugin();


PropertyPageList *pPageList;
Setup *pSetup;

// Globals

int PASCAL WinMain(HINSTANCE hInstance,  
HINSTANCE hPrevInstance,
LPSTR lpCmdLine,
int nCmdShow
)
{
	MSG msg;
	hInst = hInstance;

	DummyFunc();
	
	if (!hPrevInstance)
		if (!InitApplication(hInstance))
			return (FALSE);

	// Load setup info
	pSetup = new Setup;
	if (!pSetup->Create())
	{
		MessageBox(hWnd, "Failed to initialize.", "War setup - ERROR", MB_OK);
		delete pSetup;
		return FALSE;
	}

	if (!pSetup->LoadInfFile())
	{
		MessageBox(hWnd, "Failed to load \"setup.inf\".", "War setup - ERROR", MB_OK);
		delete pSetup;
		return FALSE;
	}
			 
	if (!InitInstance(hInstance, nCmdShow))
		return (FALSE);
	
	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg); 
		DispatchMessage(&msg);
	}
	
	// Clean up..
	if (__PostProcessPlugin)
		__PostProcessPlugin(NULL, pSetup, pPageList);
	
	return (msg.wParam);
}


BOOL InitApplication(HINSTANCE hInstance)
{
		WNDCLASSEX  wc;
		memset(&wc,0,sizeof(wc));

		wc.cbSize = sizeof(wc);
    wc.style = CS_CLASSDC;
    wc.lpfnWndProc = WndProgressProc;
    wc.hInstance = hInstance;
		wc.hIcon = LoadIcon(hInst, MAKEINTRESOURCE(IDD_SETUP));
    wc.hbrBackground = (HBRUSH)GetStockObject(LTGRAY_BRUSH);
    wc.lpszClassName = "WarInsProgress";

    if (!RegisterClassEx (&wc))
			return FALSE;

		return TRUE;
}

BOOL InitInstance(HINSTANCE hInstance,int nCmdShow)
{
	RECT ScreenRect, ClientRect;
	HWND hWndDesktop;
	if ((hWnd = CreateDialog(hInst, MAKEINTRESOURCE(IDD_SETUP),
		NULL, (DLGPROC)SetupProc)) == NULL)
	{
		int Err = GetLastError();
		return FALSE;
	}

	GetWindowRect(hWnd, &ClientRect);
	hWndDesktop = GetDesktopWindow();
	GetClientRect(hWndDesktop, &ScreenRect);
	ScreenRect.right /= 2;
	ScreenRect.right -= (ClientRect.right - ClientRect.left) / 2;
	ScreenRect.bottom -= ScreenRect.bottom / 2;
	ScreenRect.bottom -= (ClientRect.bottom - ClientRect.top) / 2;
	MoveWindow(hWnd, ScreenRect.right, ScreenRect.bottom, 
		(ClientRect.right - ClientRect.left),
		(ClientRect.bottom - ClientRect.top),
		TRUE);


	LoadPlugin();

	pPageList = new PropertyPageList(pSetup);
	
	WelcomePage *pWD = new WelcomePage(pSetup);
	if (pWD->Create(IDD_WELCOME, hWnd))
		pPageList->AddPage(pWD);

	InstallDirPage *pFP = new InstallDirPage(pSetup);
	if (pFP->Create(IDD_SETPATH, hWnd))
		pPageList->AddPage(pFP);

	InstallModePage *pIM = new InstallModePage(pSetup);
	if (pIM->Create(IDD_INSTALLMODE, hWnd))
		pPageList->AddPage(pIM);

	InstallCustomPage *pCP = new InstallCustomPage(pSetup);
	if (pCP->Create(IDD_CUSTOM_SELECT, hWnd))
		pPageList->AddPage(pCP);

	StartMenuPage *pSM = new StartMenuPage(pSetup);
	if (pSM->Create(IDD_STARTMENU, hWnd))
		pPageList->AddPage(pSM);

	BeginCopyPage *pBC = new BeginCopyPage(pSetup);
	if (pBC->Create(IDD_READY_TO_COPY, hWnd))
		pPageList->AddPage(pBC);

	CopyFilesPage *pCF = new CopyFilesPage(pSetup);
	if (pCF->Create(IDD_COPYFILES, hWnd))
		pPageList->AddPage(pCF);

	// DLL dialogs
	if (__InitPlugin)
	{
		if (!__InitPlugin(hWnd, pSetup, pPageList))
			return FALSE;
	}

	DonePage *pDP = new DonePage(pSetup);
	if (pDP->Create(IDD_DONE, hWnd))
		pPageList->AddPage(pDP);

	// Dll
	if (__PreperePlugin)
	{
		if (!__PreperePlugin(hWnd, pSetup, pPageList))
			return FALSE;
	}


	pPageList->ActivatePage();

	// See if we have some modifications to do
	if (!pSetup->m_VerboseName[0])
		strcpy(pSetup->m_VerboseName, pSetup->m_Name);

	char buf[512];
	sprintf(buf, "Setup - %s", pSetup->m_VerboseName);
	SetWindowText(hWnd, buf);

	UpdateWindow(hWnd);
	ShowWindow(hWnd, SW_SHOWNORMAL);

	// Dll
	if (__PreProcessPlugin)
	{
		if (!__PreProcessPlugin(hWnd, pSetup, pPageList))
			return FALSE;
	}

	ShowWindow(hWnd, SW_SHOWNORMAL);
	return TRUE;
}

BOOL CALLBACK SetupProc(HWND hwndDlg,
  UINT uMsg,
	WPARAM wParam,
  LPARAM lParam)
{
	LPCSTR Text = "";
	switch(uMsg)
	{
		case PropertyPage::MYWM_NEXT:
			goto move_next;
		case PropertyPage::MYWM_PREV:
			goto move_prev;
		case WM_COMMAND:
			{
				switch(wParam)
				{
					case IDC_CANCEL:
					case IDCANCEL:
						if (__OnCancelPlugin)
						{
							if (!__OnCancelPlugin(hWnd, pSetup, pPageList))
								break;
						}
						if (IsWindowEnabled(GetDlgItem(hWnd, IDC_CANCEL)))
							PostQuitMessage(-1);
						break;
					case IDC_LEFTBUTTON:
move_prev:
						pPageList->GoPrev();
						EnableWindow(GetDlgItem(hWnd, IDC_LEFTBUTTON), !pPageList->IsFirst());
						SetWindowText(GetDlgItem(hWnd, IDC_RIGHTBUTTON), "Next >>");
						break;
					case IDC_RIGHTBUTTON:
move_next:
						if (pPageList->IsLast())
						{
							// Finish.
							PostQuitMessage(0);
							break;
						}
						pPageList->GoNext();
						if (pPageList->IsLast())
							Text = "Finish";
						else
							Text = "Next >>";
						SetWindowText(GetDlgItem(hWnd, IDC_RIGHTBUTTON), Text);
						break;
					default:
						return FALSE;
				}
			}
			break;
		default:
			return FALSE;
	}
	
	return TRUE;
}


LRESULT CALLBACK WndProgressProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	static UINT s_uTaskbarRestart;
	HDC hDC;
	PAINTSTRUCT ps;
	HBRUSH hBgBrush, hFgBrush;
	RECT ClientRect, TextRect;
	static int Percent;
	static char Path[MAX_PATH];
	
	switch(uMessage)
	{
		case WM_PAINT:
			//hDC = GetDC(hWnd);
			BeginPaint(hWnd, &ps);
			hDC = ps.hdc;
			GetClientRect(hWnd, &ClientRect);
			TextRect = ClientRect;
			hBgBrush = (HBRUSH)GetStockObject(GRAY_BRUSH);
			if (Percent == 100)
				hFgBrush = (HBRUSH)CreateSolidBrush(RGB(0,255,0));
			else
				hFgBrush = (HBRUSH)GetStockObject(WHITE_BRUSH);
			
			FillRect(hDC, &ClientRect, hBgBrush);
			ClientRect.right -= (int)((float)ClientRect.right - (((float)ClientRect.right / 100) * (float)Percent));
			FillRect(hDC, &ClientRect, hFgBrush);
			SetTextColor(hDC, RGB(0,0,0));
			SetBkMode(hDC, TRANSPARENT);
			
			DrawText(hDC, 
				(LPCTSTR)Path,
				strlen(Path),
				&TextRect,
				DT_CENTER | DT_VCENTER);
			
			if (Percent == 100)
				DeleteObject(hFgBrush);
			break;
		case PropertyPage::MYWM_PROGRESS:
			Percent = wParam;
			strcpy(Path, (LPCSTR)lParam);
			Percent = (int)wParam;
			InvalidateRect(hWnd, NULL, TRUE);
			UpdateWindow(hWnd);
			break;
	}
	
	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void LoadPlugin()
{
	hPlugin = LoadLibrary(".\\WarSetup.dll");
	if (hPlugin)
	{
		// Get function pointers
		__InitPlugin = (WARPFUNC)GetProcAddress((HMODULE)hPlugin, "InitPlugin");
		__PreperePlugin = (WARPFUNC)GetProcAddress((HMODULE)hPlugin, "PreperePlugin");
		__PreProcessPlugin = (WARPFUNC)GetProcAddress((HMODULE)hPlugin, "PreProcessPlugin");
		__PostProcessPlugin = (WARPFUNC)GetProcAddress((HMODULE)hPlugin, "PostProcessPlugin");
		__OnCancelPlugin = (WARPFUNC)GetProcAddress((HMODULE)hPlugin, "OnCancelPlugin");
	}
}