
#include <windows.h>
#include <string.h>
#include <stdio.h>

#define WARSETUP_DLL __declspec(dllexport)

#include "WarSetup.h"
#include "resource.h"
#include "Dialogs.h"
#include "shlobj.h"

static int __stdcall BrowseCtrlCallback(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData);

void UpdateProgress(HWND hWnd, const char *FileName, int Progress)
{
	SendMessage(hWnd, PropertyPage::MYWM_PROGRESS, (WPARAM)Progress, (LPARAM)FileName);

/*
#ifdef _DEBUG
	Sleep(1000);
#endif
*/
}

void PumpMessages()
{
	MSG msg;
	while(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE))
	{
		if (msg.message == WM_QUIT)
			break;

		GetMessage(&msg, NULL, 0, 0);
		TranslateMessage(&msg); 
		DispatchMessage(&msg);
	}
}


PropertyPage *PropertyPage::m_pCurrent;

PropertyPage::PropertyPage(Setup *pSetup)
{
	m_hWnd = NULL;
	m_IsOK = FALSE;
	m_pOrigin = NULL;
	m_pSetup = pSetup;
	m_Name = "";
	m_bEnabled = TRUE;
}

Setup *PropertyPage::GetSetup()
{
	return m_pSetup;
}

BOOL PropertyPage::OnActivate()
{
	HWND hFrame = GetDlgItem(hWnd, IDC_DLGFRAME);

	RECT FrameRect;
	POINT FrameTop;
	GetWindowRect(hFrame, &FrameRect);
	FrameTop.x = FrameRect.left;
	FrameTop.y = FrameRect.top;
	ScreenToClient(hWnd, &FrameTop);
	GetClientRect(m_hWnd, &FrameRect);
	MoveWindow(m_hWnd,
		FrameTop.x, FrameTop.y, 
		FrameRect.right,
		FrameRect.bottom,
		TRUE);

	OnUpdate(FALSE);
	ShowWindow(m_hWnd, SW_SHOW);
	UpdateWindow(m_hWnd);
	return TRUE;
}

BOOL PropertyPage::OnProcess()
{
	return TRUE;
}

BOOL PropertyPage::OnNext()
{
	OnUpdate();
	if (!m_IsOK)
		return FALSE;

	ShowWindow(m_hWnd, SW_HIDE);
	return TRUE;
}

BOOL PropertyPage::OnPrev()
{
	OnUpdate();
	ShowWindow(m_hWnd, SW_HIDE);
	return TRUE;
}

BOOL PropertyPage::OnFinish()
{
	return TRUE;
}

BOOL PropertyPage::OnOK()
{
	if (!m_IsOK)
		return FALSE;

	OnUpdate();
	return TRUE;
}

BOOL PropertyPage::OnInitDlg()
{
	OnUpdate(FALSE);
	return TRUE;
}

BOOL PropertyPage::OnCreate()
{
	return TRUE;
}

void PropertyPage::OnUpdate(BOOL bSave)
{
	m_pOrigin->EnableNextButton(m_IsOK);
	return;
}

BOOL PropertyPage::Create(int DlgID, HWND WndParent, HINSTANCE hInstance)
{
	m_pCurrent = this;
	SetLastError(0);
	if ((m_hWnd = CreateDialog(hInstance, MAKEINTRESOURCE(DlgID),
		WndParent, (DLGPROC)PropertyProc)) == NULL)
	{
		int Err = GetLastError();
		return FALSE;
	}

	ShowWindow(m_hWnd, SW_HIDE);
	UpdateWindow(m_hWnd);
	return TRUE;
}


BOOL CALLBACK PropertyPage::PropertyProc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	PropertyPage *pDlg = (PropertyPage *)GetWindowLong(hWnd, GWL_USERDATA);
	BOOL Rval = FALSE;

	if (!pDlg)
	{
		if (m_pCurrent != NULL)
		{
			// Creation process
			pDlg = m_pCurrent;
			SetWindowLong(hWnd, GWL_USERDATA, (LONG)pDlg);
			m_pCurrent = NULL;
		}
		else
		{
			ASSERT(FALSE);
			return FALSE;
		}
	}

	pDlg->m_hWnd = hWnd;
	pDlg->m_uMessage = uMsg;
	pDlg->m_wParam = wParam;
	pDlg->m_lParam = lParam;

	if (pDlg->OnMessage(uMsg, wParam, lParam))
		return TRUE;

	switch(uMsg)
	{
		case WM_CREATE:
			pDlg->OnCreate();
			break;
		case MYWM_ACTIVATE:
			pDlg->OnActivate();
			break;
		case MYWM_OK:
			pDlg->OnOK();
			break;
		case WM_INITDIALOG:
			pDlg->OnInitDlg();
			break;
		case MYWM_UPDATE:
			pDlg->OnUpdate((BOOL)wParam);
			break;
		case MYWM_PROCESS:
			pDlg->OnProcess();
			break;
		case WM_COMMAND:
			switch(HIWORD(wParam))
			{
				case EN_CHANGE:
					pDlg->OnUpdate();
					break;
			}
			break;
		default:
			return FALSE;
	}

	return TRUE;
}

BOOL PropertyPage::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	return FALSE;
}

////////////////////////////////////////////////////////////////////////////
// PropertyPageList
////////////////////////////////////////////////////////////////////////////

PropertyPageList::PropertyPageList(Setup *pSetup)
{
	memset(m_pDlg, 0, sizeof(m_pDlg));
	m_NumPages = 0;
	m_Current = -1;
	m_pSetup = pSetup;
	m_First = 0;
}

BOOL PropertyPageList::AddPage(PropertyPage *pPage)
{
	ASSERT(m_NumPages <= MAX_PAGES);
	if (m_NumPages > MAX_PAGES)
		return FALSE;

	m_pDlg[m_NumPages++] = pPage;
	pPage->m_pOrigin = this;
	return TRUE;
}

BOOL PropertyPageList::ActivatePage(int Page)
{
	if (!m_pDlg[Page])
		return FALSE;

	ASSERT(m_pDlg[Page]->m_hWnd != NULL);

	if (!m_pDlg[Page]->OnActivate())
	{
		m_pDlg[m_Current]->OnActivate();
		return FALSE;
	}

	m_Current = Page;
	EnableWindow(GetDlgItem(hWnd, IDC_LEFTBUTTON), !IsFirst());
	EnableWindow(GetDlgItem(hWnd, IDC_RIGHTBUTTON), m_pDlg[Page]->m_IsOK);

	BOOL Rval = PostMessage(m_pDlg[m_Current]->m_hWnd, PropertyPage::MYWM_PROCESS, 0, 0);
	return TRUE;
}

BOOL PropertyPageList::GoNext()
{
	if (!m_pDlg[m_Current]->OnOK()
		|| !m_pDlg[m_Current]->OnNext())
		return FALSE;

	for(int i = m_Current +1; i < m_NumPages; i++)
	{
		if (m_pDlg[i]->m_bEnabled)
			return ActivatePage(i);
	}

	return FALSE;
}

BOOL PropertyPageList::GoPrev()
{
	if (!m_pDlg[m_Current]->OnPrev())
		return FALSE;

	for(int i = m_Current - 1; i >= m_First ; i--)
	{
		if (m_pDlg[i]->m_bEnabled)
			return ActivatePage(i);
	}

	return NULL;
}

PropertyPage *PropertyPageList::GetPage(LPCSTR Name)
{
	for(int i = 0; i < m_NumPages; i++)
	{
		if (!stricmp(Name, m_pDlg[i]->m_Name))
			return m_pDlg[i];
	}

	return NULL;
}


BOOL PropertyPageList::IsEnabled(LPCSTR Name)
{
	PropertyPage *p = GetPage(Name);
	return p ? p->m_bEnabled : FALSE;
}

BOOL PropertyPageList::Enable(LPCSTR Name, BOOL bEnable)
{
	PropertyPage *p = GetPage(Name);
	if (!p)
		return FALSE;

	p->m_bEnabled = bEnable;
	return TRUE;
}

BOOL PropertyPageList::IsFirst()
{
	return GetCurr() == GetFirst();
}

BOOL PropertyPageList::IsLast() 
{
	return GetCurr() == GetLast();
}

PropertyPage *PropertyPageList::GetFirst()
{
	for(int i = m_First; i < m_NumPages; i++)
	{
		if (m_pDlg[i]->m_bEnabled)
			return m_pDlg[i];
	}

	return NULL;
}

PropertyPage *PropertyPageList::GetLast()
{
	for(int i = m_NumPages - 1; i >= m_First ; i--)
	{
		if (m_pDlg[i]->m_bEnabled)
			return m_pDlg[i];
	}

	return NULL;
}

PropertyPage *PropertyPageList::GetCurr()
{
	if (m_Current < 0)
		return NULL;

	return m_pDlg[m_Current];
}

void PropertyPageList::EnableCancelButton(BOOL bEnable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_CANCEL), bEnable);
}

void PropertyPageList::EnableNextButton(BOOL bEnable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_RIGHTBUTTON), bEnable);
}

void PropertyPageList::EnablePrevButton(BOOL bEnable)
{
	EnableWindow(GetDlgItem(hWnd, IDC_LEFTBUTTON), bEnable);
}

BOOL PropertyPageList::SetAsFirst(LPCSTR Name)
{
	for(int i = 0; i < m_NumPages; i++)
	{
		if (!stricmp(Name, m_pDlg[i]->m_Name))
		{
			m_First = i;
			return TRUE;
		}
	}

	return FALSE;
}



////////////////////////////////////////////////////////////////////////////
// Welcome dialog
////////////////////////////////////////////////////////////////////////////

WelcomePage::WelcomePage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_hIcon = NULL;
	m_Name = "WelcomePage";
}

BOOL WelcomePage::OnInitDlg()
{
	PropertyPage::OnInitDlg();
	
	m_IsOK = TRUE;

	char buf[1024];
	char newbuf[1024];
	GetWindowText(GetDlgItem(m_hWnd, IDC_TEXT), buf, sizeof(buf));
	LPSTR p = buf;
	while(*p)
	{
		if (*p == '$')
		{
			*p = 0;
			strcpy(newbuf, buf);
			strcat(newbuf, GetSetup()->m_VerboseName);
			strcat(newbuf, ++p);
			SetWindowText(GetDlgItem(m_hWnd, IDC_TEXT), newbuf);
			break;
		}
		++p;
	}

	if (GetFileAttributes("readme.txt") != 0xffffffff)
		ShowWindow(GetDlgItem(m_hWnd, IDC_README), SW_SHOW);

	//ExtractIcon
	if (GetSetup()->m_PrimaryTarget[0])
	{
		m_hIcon = ExtractIcon(hInst, GetSetup()->m_PrimaryTarget, 0);
		if (m_hIcon)
			SendMessage(GetDlgItem(m_hWnd, IDC_TARGETICON), 
				STM_SETICON , (WPARAM)m_hIcon, 0);
	}

	return TRUE;
}

BOOL WelcomePage::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (BUTTON(IDC_README))
	{
		ShellExecute(m_hWnd, "open", "readme.txt", NULL, "", SW_SHOW);
		return TRUE;
	}

	return PropertyPage::OnMessage(uMsg, wParam, lParam);
}

////////////////////////////////////////////////////////////////////////////
// InstallDirPage dialog
////////////////////////////////////////////////////////////////////////////

InstallDirPage::InstallDirPage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "InstallDirPage";
	m_Path[0] = 0;
	m_IsOK = TRUE;
}

BOOL InstallDirPage::OnInitDlg()
{
	sprintf(m_Path,"%s\\%s", GetSetup()->ProgramDir, GetSetup()->m_InstallDir); 
	return PropertyPage::OnInitDlg();
}

BOOL InstallDirPage::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (BUTTON(IDC_BROWSE))
	{
		OnBrowse();
		return TRUE;
	}

	return PropertyPage::OnMessage(uMsg, wParam, lParam);
}

BOOL InstallDirPage::OnOK()
{
	OnUpdate();
	
	return PropertyPage::OnOK();
}

void InstallDirPage::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_PATH), m_Path, sizeof(m_Path));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_PATH), m_Path);
	}

	m_IsOK = TRUE;
	DWORD Flags = GetFileAttributes(m_Path);
	if ((Flags != 0xffffffff)
		&& (Flags & FILE_ATTRIBUTE_READONLY))
		m_IsOK = FALSE;

	PropertyPage::OnUpdate();
}

void InstallDirPage::OnBrowse()
{
	LPMALLOC pMalloc;
  if (SHGetMalloc (&pMalloc)!= NOERROR)
		return;

  BROWSEINFO bInfo;
  LPITEMIDLIST pidl;
  ZeroMemory((PVOID) &bInfo,sizeof (BROWSEINFO));

  if (FALSE && m_Path[0])
  {
    OLECHAR       olePath[MAX_PATH];
    ULONG         chEaten;
    ULONG         dwAttributes;
    HRESULT       hr;
    LPSHELLFOLDER pDesktopFolder;
		if (SUCCEEDED(SHGetDesktopFolder(&pDesktopFolder)))
		{
      MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, m_Path, -1, olePath, MAX_PATH);
      hr = pDesktopFolder->ParseDisplayName(NULL,
				NULL, olePath, &chEaten, &pidl, &dwAttributes);
      if (FAILED(hr))
      {
        pMalloc ->Free (pidl);
        pMalloc ->Release ();
        return;
      }
      bInfo.pidlRoot = pidl;
    }
  }
  
	bInfo.hwndOwner = m_hWnd;
  bInfo.pszDisplayName = m_Path;
  bInfo.lpszTitle = "Select a location to install the files";
  bInfo.ulFlags = BIF_RETURNFSANCESTORS|BIF_RETURNONLYFSDIRS ;
  bInfo.lpfn = BrowseCtrlCallback;  // address of callback function
  bInfo.lParam = (LPARAM)this;      // pass address of object to callback function

  if ((pidl = ::SHBrowseForFolder(&bInfo)) == NULL)
  {
    return;
  }

  if (::SHGetPathFromIDList(pidl,m_Path) == FALSE)
  {
    pMalloc ->Free(pidl);
    pMalloc ->Release();
    return;
  }

  pMalloc ->Free(pidl);
  pMalloc ->Release();

	OnUpdate(FALSE);
}

// after initialization and when selection changes
static int __stdcall BrowseCtrlCallback(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	InstallDirPage *p = (InstallDirPage *)lpData;
  if (uMsg == BFFM_INITIALIZED && p->m_Path[0])
  {
    ::SendMessage(hwnd, BFFM_SETSELECTION, TRUE, (LPARAM)(LPCTSTR)p->m_Path);
  }
  else // uMsg == BFFM_SELCHANGED
  {
  }

  return 0;
}


/////////////////////////////////////////////////////////////////////////////
// InstallModePage
/////////////////////////////////////////////////////////////////////////////


InstallModePage::InstallModePage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "InstallModePage";
	m_Mode = mode_normal;
	m_IsOK = TRUE;
}


BOOL InstallModePage::OnInitDlg()
{
	return PropertyPage::OnInitDlg();
}

void InstallModePage::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		for(int i = 0; i < (int)mode_invalid; i++)
		{
			HWND hw = GetDlgItem(m_hWnd, IDC_RADIO1 + i);
			if (SendMessage(hw, BM_GETCHECK, 0, 0) == BST_CHECKED)
			{
				m_Mode = (InstallModesDef) i;
				break;
			}
		}	
	}
	else
	{
		for(int i = 0; i < (int)mode_invalid; i++)
		{
			HWND hw = GetDlgItem(m_hWnd, IDC_RADIO1 + i);
			SendMessage(hw, BM_SETCHECK, (m_Mode == (InstallModesDef) i) 
				? BST_CHECKED : BST_UNCHECKED, 0);
		}	
	}

	PropertyPage::OnUpdate();
}

BOOL InstallModePage::OnOK()
{
	OnUpdate();
	return m_pOrigin->Enable("InstallCustomPage", m_Mode == mode_custom);
}

/////////////////////////////////////////////////////////////////////////////
// InstallCustomPage
/////////////////////////////////////////////////////////////////////////////


InstallCustomPage::InstallCustomPage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "InstallCustomPage";
	m_IsOK = TRUE;
	memset(&m_Data, 0, sizeof(m_Data));
}


BOOL InstallCustomPage::OnInitDlg()
{
	PropertyPage::OnInitDlg();
	// Set up selections
	int i = 0;
	for (CClassList *p = GetSetup()->m_FileClassRoot
		; p && (i < NUM_MODULES)
		; p = p->m_Next, i++)
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_CHECKMODULE + i), (LPCSTR)p->m_Name);
		ShowWindow(GetDlgItem(m_hWnd, IDC_CHECKMODULE + i), SW_SHOW);
		ShowWindow(GetDlgItem(m_hWnd, IDC_QUERYFILES + i), SW_SHOW);
		m_Data[i].m_bSelected = p->m_IsTypical;
		m_Data[i].m_pClass = p;
		SetState(i);
	}

	return TRUE;
}

void InstallCustomPage::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		for(int i = 0; i < NUM_MODULES; i++)
		{
			GetState(i);
		}
	}
	else
	{
		for(int i = 0; i < NUM_MODULES; i++)
		{
			SetState(i);
		}
	}
	PropertyPage::OnUpdate();
}

void InstallCustomPage::SetState(int Index)
{
	if ((Index >= NUM_MODULES) || (Index < 0))
		return;

	SendMessage(GetDlgItem(m_hWnd, IDC_CHECKMODULE + Index),
		BM_SETCHECK, m_Data[Index].m_bSelected, 0);
}

BOOL InstallCustomPage::GetState(int Index)
{
	if ((Index >= NUM_MODULES) || (Index < 0))
		return FALSE;

	return m_Data[Index].m_bSelected = SendMessage(GetDlgItem(m_hWnd, IDC_CHECKMODULE + Index),
		BM_GETCHECK, 0, 0);
}

BOOL InstallCustomPage::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	CClassList *p = NULL;	
	if BUTTON(IDC_QUERYFILES) 
		p = m_Data[0].m_pClass;
	if BUTTON(IDC_QUERYFILES + 1) 
		p = m_Data[1].m_pClass;
	if BUTTON(IDC_QUERYFILES + 2) 
		p = m_Data[2].m_pClass;
	if BUTTON(IDC_QUERYFILES + 3) 
		p = m_Data[3].m_pClass;
	if BUTTON(IDC_QUERYFILES + 4) 
		p = m_Data[4].m_pClass;
	if BUTTON(IDC_QUERYFILES + 5) 
		p = m_Data[5].m_pClass;
	if BUTTON(IDC_QUERYFILES + 6) 
		p = m_Data[6].m_pClass;
	if BUTTON(IDC_QUERYFILES + 7) 
		p = m_Data[7].m_pClass;

	if (p)
	{
		FileList Dlg(GetSetup(), p);
		Dlg.DoModal(m_hWnd);
		return TRUE;
	}

	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////
// StartMenuPage
///////////////////////////////////////////////////////////////////////////////

StartMenuPage::StartMenuPage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "StartMenuPage";
	m_IsOK = TRUE;
	m_StartMenu[0] = 0;
	m_bAddStartMenu = TRUE;
}

BOOL StartMenuPage::OnInitDlg()
{
	PropertyPage::OnInitDlg();
	// Get a list of start-menus...
	if ((GetSetup()->OsType == OS_WIN95) || (GetSetup()->OsType == OS_NT4))
	{
		// Get start groups - use user selection for now...
		HANDLE h;
		WIN32_FIND_DATA Data;
		char Path[MAX_PATH];
		sprintf(Path, "%s\\*", GetSetup()->UserProgramFolderDir);

		if ((LPARAM)GetSetup()->m_Menu[0])
			SendMessage(GetDlgItem(m_hWnd, IDC_LIST1), LB_ADDSTRING, 0, 
				(LPARAM)GetSetup()->m_Menu);

		if ((h = FindFirstFile(Path,&Data)) != INVALID_HANDLE_VALUE)
		{
			do
			{
				if (!(Data.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
					continue;
				if (!strcmp(Data.cFileName,".") || !strcmp(Data.cFileName,".."))
					continue;

				SendMessage(GetDlgItem(m_hWnd, IDC_LIST1), LB_ADDSTRING, 0, (LPARAM)Data.cFileName);

			} while(FindNextFile(h,&Data));
			FindClose(h);
		}
	}
	if (GetSetup()->OsType == OS_NT351)
	{
		GetSetup()->DDEGroupRetrieval(GetDlgItem(m_hWnd, IDC_LIST1));
	}

	SendMessage(GetDlgItem(m_hWnd, IDC_LIST1), LB_SELECTSTRING , -1, 
		(LPARAM)GetSetup()->m_Menu);

	return TRUE;
}

void StartMenuPage::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		int Index = SendMessage(GetDlgItem(m_hWnd,IDC_LIST1), LB_GETCURSEL, 0 , 0);
		SendMessage(GetDlgItem(m_hWnd,IDC_LIST1), LB_GETTEXT, WPARAM(Index), (LPARAM)m_StartMenu);
		m_bAddStartMenu = SendMessage(GetDlgItem(m_hWnd, IDC_CHECK1),
			BM_GETCHECK, 0, 0);
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd,IDC_LIST1), (LPCSTR)m_StartMenu);
		SendMessage(GetDlgItem(m_hWnd, IDC_CHECK1),
			BM_SETCHECK, m_bAddStartMenu, 0);
	}

	EnableWindow(GetDlgItem(m_hWnd, IDC_LIST1), m_bAddStartMenu);
	PropertyPage::OnUpdate();
}

BOOL StartMenuPage::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (BUTTON(IDC_CHECK1))
	{
		if (HIWORD(wParam) == BN_CLICKED)
			OnUpdate();
		return TRUE;
	}

	return PropertyPage::OnMessage(uMsg, wParam, lParam);
}


///////////////////////////////////////////////////////////////////////////////
// FileList
///////////////////////////////////////////////////////////////////////////////

FileList *FileList::m_pCurrent;
FileList::FileList(Setup *pSetup, CClassList *pClass)
{
	m_pSetup = pSetup;
	m_pClass = pClass;
	m_pCurrent = NULL;
}

void FileList::DoModal(HWND hParent)
{
	m_pCurrent = this;
	DialogBox(hInst, MAKEINTRESOURCE(IDD_FILELIST), /*hParent*/GetParent(hParent), (DLGPROC)FileListPrc);
	//SetForegroundWindow(hParent);
	//ShowWindow(GetParent(hParent), SW_SHOW);
	m_pCurrent = NULL;
}

BOOL CALLBACK FileList::FileListPrc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if BUTTON(IDCANCEL)
	{
		EndDialog(hWnd, IDCANCEL);
		return TRUE;
	}
		
	switch(LOWORD(uMsg))
	{
		case WM_INITDIALOG:
			m_pCurrent->OnInitdialog(hWnd);
			return TRUE;
	}

	return FALSE;
}

BOOL FileList::OnInitdialog(HWND hWnd)
{
	LVCOLUMN ch;
	memset(&ch, 0, sizeof(ch));
	int Index = 0;

	ch.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_ORDER;
	ch.fmt = LVCFMT_LEFT;
	ch.cx = 200;
	ch.pszText = "Name";
	ch.cchTextMax = strlen(ch.pszText);
	ch.iSubItem = Index;
	ch.iOrder = Index;
	SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_INSERTCOLUMN, Index, (LPARAM)&ch);

	ch.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_ORDER;
	ch.fmt = LVCFMT_RIGHT;
	ch.cx = 60;
	ch.pszText = "Bytes";
	ch.cchTextMax = strlen(ch.pszText);
	ch.iSubItem = ++Index;
	ch.iOrder = Index;
	SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_INSERTCOLUMN, Index, (LPARAM)&ch);

	ch.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_ORDER;
	ch.fmt = LVCFMT_LEFT;
	ch.cx = 70;
	ch.pszText = "Version";
	ch.cchTextMax = strlen(ch.pszText);
	ch.iSubItem = ++Index;
	ch.iOrder = Index;
	SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_INSERTCOLUMN, Index, (LPARAM)&ch);

	ch.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_ORDER;
	ch.fmt = LVCFMT_LEFT;
	ch.cx = 150;
	ch.pszText = "Provider";
	ch.cchTextMax = strlen(ch.pszText);
	ch.iSubItem = ++Index;
	ch.iOrder = Index;
	SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_INSERTCOLUMN, Index, (LPARAM)&ch);

	ch.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM | LVCF_ORDER;
	ch.fmt = LVCFMT_LEFT;
	ch.cx = 150;
	ch.pszText = "Comments";
	ch.cchTextMax = strlen(ch.pszText);
	ch.iSubItem = ++Index;
	ch.iOrder = Index;
	SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_INSERTCOLUMN, Index, (LPARAM)&ch);


	Index = 0;
	DWORD TotalSize = 0;
	char buf[32];
	for(CFileList *p = m_pClass->m_FileList; p; p = p->m_Next)
	{
		LVITEM li;
		memset(&li, 0, sizeof(li));
		li.mask = LVIF_TEXT;
		li.iItem = Index++;
		li.iSubItem = 0;
		li.pszText = p->m_DstName;
		li.cchTextMax =	strlen(p->m_DstName);

		int ItemIndex = SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_INSERTITEM, 0, (LPARAM)&li);

		// See if we can get the size of the file too..
		LPCSTR Name = strrchr(p->m_SrcName, '\\');
		if (Name)
			++Name;
		else
			Name = p->m_SrcName;
		HANDLE h = CreateFile(Name, 0, 0, NULL, OPEN_EXISTING, 0, NULL);
		if (h != (HANDLE)0xffffffff)
		{
			DWORD Size = GetFileSize(h, NULL);
			CloseHandle(h);

			TotalSize += Size;
			sprintf(buf,"%d", Size);

			li.mask = LVIF_TEXT;
			li.iItem = ItemIndex;
			li.iSubItem = 1;
			li.pszText = buf;
			li.cchTextMax =	strlen(p->m_DstName);
			SendMessage(GetDlgItem(hWnd, IDC_LIST1), LVM_SETITEM, 0, (LPARAM)&li);
		}

		m_pSetup->QueryFileInfoForListCntr(Name, GetDlgItem(hWnd, IDC_LIST1), 
			ItemIndex, 2, "ProductVersion", "CompanyName", "Comments", NULL);
	}

	sprintf(buf, "%d kbytes", TotalSize /= 1024);
	SetWindowText(GetDlgItem(hWnd, IDC_KBYTES), buf);
	SetWindowText(hWnd, m_pClass->m_Name);

	return TRUE;
}


///////////////////////////////////////////////////////////////////////////////
// BeginCopyPage
///////////////////////////////////////////////////////////////////////////////

BeginCopyPage::BeginCopyPage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "BeginCopyPage";
	m_IsOK = TRUE;
}

///////////////////////////////////////////////////////////////////////////////
// CopyFilesPage
///////////////////////////////////////////////////////////////////////////////

CopyFilesPage::CopyFilesPage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "CopyFilesPage";
	m_IsOK = TRUE;
}

BOOL CopyFilesPage::OnActivate()
{
	PropertyPage::OnActivate();
	m_pOrigin->SetAsFirst("CopyFilesPage");
	return TRUE;
}

BOOL CopyFilesPage::OnProcess()
{
	PropertyPage::OnProcess();

	// Install.
	InstallDirPage *pIP = (InstallDirPage *)m_pOrigin->GetPage("InstallDirPage");
	ASSERT(pIP != NULL);
	strcpy(m_pSetup->DestPath,pIP->m_Path);

	StartMenuPage *pSM = (StartMenuPage *)m_pOrigin->GetPage("StartMenuPage");
	ASSERT(pSM != NULL); 
	if (pSM->m_bAddStartMenu)
		strcpy(m_pSetup->FolderName, pSM->m_StartMenu);
	else
		m_pSetup->FolderName[0] = 0; // Do not create start menu items

	InstallModePage *pIM = (InstallModePage *)m_pOrigin->GetPage("InstallModePage");
	ASSERT(pIM != NULL);
	DWORD Mask = 0;

	if (pIM->m_Mode == InstallModePage::mode_custom)
	{
		InstallCustomPage *pCP = (InstallCustomPage  *)m_pOrigin->GetPage("InstallCustomPage");
		
		for(int i =0; i < InstallCustomPage::NUM_MODULES; i++)
		{
			if (pCP->m_Data[i].m_bSelected)
				Mask |= (1 << i);
		}
	}

	if (!m_pSetup->InstallSelection(GetDlgItem(m_hWnd, IDC_PROGRESS),
		GetDlgItem(m_hWnd, IDC_CLASS), 
		(int)pIM->m_Mode, Mask))
	{
		m_pOrigin->SetAsFirst("WelcomePage");
		m_IsOK = FALSE;
		MessageBox(NULL, "Failed to install. Please verify the install properties.", "Error", MB_OK);
		m_pOrigin->EnableNextButton(FALSE);
		PostQuitMessage(-1);
	}

	// Move next
	PostMessage(GetParent(m_hWnd), MYWM_NEXT, 0, 0);
	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
// DonePage
///////////////////////////////////////////////////////////////////////////////

DonePage::DonePage(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "DonePage";
	m_IsOK = TRUE;
}

BOOL DonePage::OnActivate()
{
	PropertyPage::OnActivate();

	// Disable previous dialogs
	m_pOrigin->SetAsFirst("DonePage");

	// Disable cancel button
	m_pOrigin->EnableCancelButton(FALSE);
	m_pOrigin->EnablePrevButton(FALSE);

	return TRUE;
}

BOOL DonePage::OnInitDlg()
{
	PropertyPage::OnInitDlg();

	char buf[1024];
	char newbuf[1024];
	GetWindowText(GetDlgItem(m_hWnd, IDC_INFO), buf, sizeof(buf));
	LPSTR p = buf;
	while(*p)
	{
		if (*p == '$')
		{
			*p = 0;
			strcpy(newbuf, buf);
			strcat(newbuf, GetSetup()->m_VerboseName);
			strcat(newbuf, ++p);
			SetWindowText(GetDlgItem(m_hWnd, IDC_INFO), newbuf);
			break;
		}
		++p;
	}	

	return TRUE;
}


