
#include <windows.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>

#include "resource.h"

#define WARSETUP_DLL __declspec(dllexport)

#include "WarSetup.h"
#include "SetupFiles.h"
#include "winreg.h"
#include "winver.h"
#include "winnetwk.h"
#include "objbase.h"
#include <ShlObj.h>
#include "winnls.h"

void WarSetupSharedPCH();

Setup::Setup()
{
	WinDir[0] = 0;
	SystemDir[0] = 0;
	ProgramFolderDir[0] = 0;
	UserProgramFolderDir[0] = 0;
	ProgramDir[0] = 0;
	StartUpMenu[0] = 0;
	DestPath[0] = 0;
	m_Menu[0] = 0;
	m_Name[0] = 0;
	m_InstallDir[0] = 0;
	FolderName[0] = 0;
	m_VerboseName[0] = 0;
	int  OsType = OS_UNKNOWN;
	m_LogFp = NULL;
	m_UnInstFp = NULL;
	m_DoReboot = FALSE;
	BOOL m_NT351 = FALSE;
	BOOL m_NT4 = FALSE;
	BOOL m_Win95 = FALSE;
	m_Copyright[0] = 0;
	m_InstallDir[0] = 0;
	m_CompanyName[0] = 0;
	m_Comments[0] = 0;
	m_SpecialBuild[0] = 0;
	m_LegalTrademarks[0] = 0;
	m_Version[0] = 0;
	m_PrimaryTarget[0] = 0;
	m_FileClassRoot = NULL;
	WarSetupSharedPCH(); // Dummy func
}

Setup::~Setup()
{
}


int Setup::MySscanf(LPCSTR buf, LPCSTR Format, ...)
{
	va_list marker;
	va_start(marker, Format);
	int Rval = 0;
	char *p;
	int *i;
	int base;
	
	while(*Format)
	{
		if (*Format == '%')
		{
			switch(*++Format)
			{
				case 's':
					p = va_arg(marker,char *);
					while(*buf && (!Format[1] || (*buf != Format[1])))
						*p++ = *buf++;
					*p = 0;
					break;
				case 'l':
					++Format; // %ld, skip over one to get in sync
				case 'O':
					base = 10;
					i = va_arg(marker, int *);
					*i = 0;
					if (*buf == '0')
						base = 8; // Octal
					while(*buf && isdigit(*buf))
					{
						*i *= base;
						*i += *buf - '0';
						++buf;
					}
					break;
				case 'd':
					base = 10;
					i = va_arg(marker, int *);
					*i = 0;
					while(*buf && isdigit(*buf))
					{
						*i *= base;
						*i += *buf - '0';
						++buf;
					}
					break;
			}
			++Format;
		}
		else
		{
			while(*buf && (*buf != *Format))
				++buf;
			++Format;
			++buf;
		}
	}

	va_end(marker);
	return Rval;
}

LPCSTR Setup::StripPath(LPCSTR Path)
{
	LPCSTR Name;

	if ((Name = strrchr(Path,'\\')) != NULL)
		++Name;
	else
		Name = Path;

	return Name;
}

BOOL Setup::Create()
{
	// Check OS and environment
	OsInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
	GetVersionEx(&OsInfo);
	if (OsInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
	{
		OsType = OS_WIN95;
	}
	else if (OsInfo.dwPlatformId == VER_PLATFORM_WIN32_NT)
	{
		if (OsInfo.dwMajorVersion >= 4)
			OsType = OS_NT4;
		else
			OsType = OS_NT351; // Put some more checking here in the future
	}

	if (OsType == OS_UNKNOWN)
	{
		MyMessageBox("Unknown version of Windows. Install aborted.");
		return FALSE;
	}

	// Initialize usful globals...
	switch(OsType)
	{
		case OS_WIN95: if (!LoadWin95Info()) return FALSE; break;
		case OS_NT4: if (!LoadNT4Info()) return FALSE;	break;
		case OS_NT351: if (!LoadNt351Info()) return FALSE; break;
	}

	return TRUE;
}

BOOL Setup::LoadNT4Info()
{
	if (!GetRegisteryString(
		HKEY_LOCAL_MACHINE,
		"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ProgramFilesDir",
		ProgramDir,
		sizeof(ProgramDir)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get NT4 Program dir\n");
		MyMessageBox("Failed to get registery value for ProgramDir");
		*ProgramDir = 0;
	}

	if (!GetRegisteryString(
		HKEY_LOCAL_MACHINE,
		"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders\\Common Programs",
		ProgramFolderDir,
		sizeof(ProgramFolderDir)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get NT4 ProgramFolder dir\n");
		MyMessageBox("Failed to get registery value for ProgramFolderDir");
		*ProgramFolderDir = 0;
	}

	if (!GetRegisteryString(
		HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders\\Programs",
		UserProgramFolderDir,
		sizeof(UserProgramFolderDir)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get NT4 UserProgramFolder dir\n");
		MyMessageBox("Failed to get registery value for UserProgramFolderDir");
		*UserProgramFolderDir = 0;
	}

	if (!GetRegisteryString(
		HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\User Shell Folders\\Startup",
		StartUpMenu,
		sizeof(StartUpMenu)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get NT4 UserStartupFolder dir\n");
		MyMessageBox("Failed to get registery value for UserProgramFolderDir");
		*UserProgramFolderDir = 0;
	}

	GetWindowsDirectory(WinDir,sizeof(WinDir));
	GetSystemDirectory(SystemDir,sizeof(SystemDir));

	return TRUE;
}

BOOL Setup::LoadWin95Info()
{
	if (!GetRegisteryString(
		HKEY_LOCAL_MACHINE,
		"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\ProgramFilesDir",
		ProgramDir,
		sizeof(ProgramDir)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get Win95 Program dir\n");
		MyMessageBox("Failed to get registery value for ProgramDir");
		*ProgramDir = 0;
	}

	if (!GetRegisteryString(
		HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders\\Programs",
		UserProgramFolderDir,
		sizeof(UserProgramFolderDir)))
	{
		DWORD err = GetLastError();
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get Win95 UserProgramFolder dir\n");
		MyMessageBox("Failed to get registery value for UserProgramFolderDir");
		*UserProgramFolderDir = 0;
	}

	
	if (!GetRegisteryString(
		HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders\\Programs",
		ProgramFolderDir,
		sizeof(ProgramFolderDir)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get Win95 ProgramFolder dir\n");
		MyMessageBox("Failed to get registery value for ProgramFolderDir");
		*ProgramFolderDir = 0;
	}

	if (!GetRegisteryString(
		HKEY_CURRENT_USER,
		"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders\\Startup",
		StartUpMenu,
		sizeof(StartUpMenu)))
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed to get Win95 Startup folder dir\n");
		MyMessageBox("Failed to get registery value for ProgramFolderDir");
		*ProgramFolderDir = 0;
	}

	GetWindowsDirectory(WinDir,sizeof(WinDir));
	GetSystemDirectory(SystemDir,sizeof(SystemDir));

	return TRUE;
}


BOOL Setup::LoadNt351Info()
{
	GetWindowsDirectory(WinDir,sizeof(WinDir));
	GetSystemDirectory(SystemDir,sizeof(SystemDir));
	sprintf(ProgramDir,"%c:", *WinDir);
	return TRUE;
}


BOOL Setup::GetRegisteryString(HKEY MainKey, LPCSTR Path, LPSTR Buf, size_t Buflen)
{
	DWORD dwNihl, dwSize = Buflen;
	LONG Rval;
	HKEY hkey;
	char szKey[MAX_PATH];
	LPSTR Tag;

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Trying to get registery key: %s ", Path);

	strcpy(szKey, Path);
	Tag = strrchr(szKey,'\\');
	ASSERT(Tag && (*Tag == '\\'));
	*Tag = 0;

	Tag = strrchr(Path,'\\');
	++Tag;

	if ((Rval = RegOpenKeyEx(
		MainKey,
		szKey,
		0,
		KEY_QUERY_VALUE,
		&hkey)) == ERROR_SUCCESS) 
	{
		if ((Rval = RegQueryValueEx(
			hkey,
			Tag,
			NULL,
			&dwNihl,
			(LPBYTE)Buf,
			&dwSize)) == ERROR_SUCCESS)
		{
			if (dwNihl == REG_EXPAND_SZ)
			{
				char *MyBuf = new char[Buflen];
				if (ExpandEnvironmentStrings(Buf,MyBuf,Buflen))
					strcpy(Buf,MyBuf);
				delete MyBuf;
			}
			RegCloseKey(hkey);
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"OK\n");
			return TRUE;
		}
		RegCloseKey(hkey);
	}

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed\n");
	return FALSE;
}

BOOL Setup::SetRegisteryString(HKEY MainKey, LPCSTR Key, LPCSTR Buf, size_t Buflen)
{
	DWORD dwSize = Buflen;
	LONG Rval;
	HKEY hkey;
	char szKey[MAX_PATH];
	LPSTR Tag;

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Trying to set registery key: %s ", Key);

	strcpy(szKey, Key);
	Tag = strrchr(szKey,'\\');
	ASSERT(Tag && (*Tag == '\\'));
	*Tag = 0;

	Tag = strrchr(Key,'\\');
	++Tag;

	if (RegOpenKeyEx(
		MainKey,
		szKey,
		0,
		KEY_WRITE,
		&hkey) == ERROR_SUCCESS) 
	{
		if ((Rval = RegSetValueEx(
			hkey,
			Tag,
			0,
			REG_SZ,
			(LPBYTE)Buf,
			dwSize)) == ERROR_SUCCESS)
		{
			fprintf(m_UnInstFp,"key %s %s\n", 
				(MainKey == HKEY_CLASSES_ROOT) ? "HKEY_CLASSES_ROOT"
				: (MainKey == HKEY_CURRENT_USER) ? "HKEY_CURRENT_USER"
				: (MainKey == HKEY_LOCAL_MACHINE) ? "HKEY_LOCAL_MACHINE" : "HKEY_USERS",
				Key);
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"OK\n");
			RegCloseKey(hkey);
			return TRUE;
		}
		RegCloseKey(hkey);
	}

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed\n");
	return FALSE;
}


BOOL Setup::CreateRegisteryKey(HKEY MainKey, LPCSTR Key)
{
	DWORD dwNihl;
	LONG Rval;
	HKEY hkey;
	

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Trying to create registery key: %s ", Key);

	if ((Rval = RegCreateKeyEx(
		MainKey,
		Key,
		0,
		"",
		REG_OPTION_NON_VOLATILE,
		KEY_SET_VALUE,
		NULL,
		&hkey,
		&dwNihl)) == ERROR_SUCCESS)
	{
		RegCloseKey(hkey);
		fprintf(m_UnInstFp,"key %s %s\n", 
			(MainKey == HKEY_CLASSES_ROOT) ? "HKEY_CLASSES_ROOT"
			: (MainKey == HKEY_CURRENT_USER) ? "HKEY_CURRENT_USER"
			: (MainKey == HKEY_LOCAL_MACHINE) ? "HKEY_LOCAL_MACHINE" : "HKEY_USERS",
			Key);
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"OK\n");
		return TRUE;
	}
	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"**Failed\n");
	return FALSE;
}

BOOL Setup::LoadInfFile()
{
	FILE *fp;
	char ReadBuf[MAX_PATH * 3];
	char buf[MAX_PATH * 2];
	int Version = 0;
	CClassList *cl;
	CFileList *fl;
	BOOL IsScanningClass = FALSE;
	int Index = LB_ERR;
	int Ofs;

	if ((fp = fopen("Setup.inf","r")) == NULL)
	{
		MyMessageBox("Failed to open file.");
		return FALSE;
	}

	// Read header line
	fgets(ReadBuf,sizeof(ReadBuf),fp);
	MySscanf(ReadBuf,"\1%s\1 %d", buf, &Version);

	if (Version < 1)
	{
		MyMessageBox("Unknown .inf file format.");
		fclose(fp);
		return FALSE;
	}

	
	Get(m_Name,sizeof(m_Name),fp);
	Get(m_Menu,sizeof(m_Menu),fp);
	
	if (Version > 1)
		Get(m_InstallDir,sizeof(m_InstallDir),fp);

	if (Version > 2)
		Get(m_VerboseName,sizeof(m_VerboseName),fp);

	fgets(ReadBuf,sizeof(ReadBuf),fp);
	MySscanf(ReadBuf,"%d %d %d", &m_NT351, &m_NT4, &m_Win95);

	switch(OsType)
	{
		case OS_WIN95:
			if (!m_Win95)
			{
				MyMessageBox("This program can not be installed on Windows 95");
				return FALSE;
			}
			break;
		case OS_NT4:
			if (!m_NT4)
			{
				MyMessageBox("This program can not be installed on NT 4");
				return FALSE;
			}
			break;
		case OS_NT351: 
			if (!m_NT351)
			{
				MyMessageBox("This program can not be installed on NT 3.51");
				return FALSE;
			}
			break;
	}	

	// Scan for file classes
	while(fgets(ReadBuf,sizeof(ReadBuf),fp) != NULL)
	{
		if (*ReadBuf == '@')
		{
			Ofs = 1;
			if (ReadBuf[1] == '@')
				Ofs = 2;

			if (ReadBuf[Ofs] == 1)
			{
				IsScanningClass = TRUE;
				// Accept this class
				cl = new CClassList;
				cl->m_Next = NULL;
				cl->m_FileList = NULL;
				
				MySscanf(ReadBuf +Ofs,"\1%s\1 %d %d %d %d %d %d %d \1%s\1\n",
				cl->m_Name,
				&cl->m_DoNT351,
				&cl->m_DoNT4,
				&cl->m_DoWin95,
				&cl->m_IsShared,
				&cl->m_IsSystem,
				&cl->m_IsTypical,
				&cl->m_IsMinimum,
				&cl->m_InstallDir);

				switch(OsType)
				{
					case OS_WIN95:
						if (!cl->m_DoWin95)
						{
skip:					
							delete cl;
							IsScanningClass = FALSE;
							continue;
						}
						break;
					case OS_NT4:
						if (!cl->m_DoNT4)
							goto skip;
						break;
					case OS_NT351: 
						if (!cl->m_DoNT351)
							goto skip;
						break;
				}	

				// We probarbly need these files. Link to global list
				cl->m_Next = m_FileClassRoot;
				m_FileClassRoot = cl;
			}
			else
				IsScanningClass = FALSE;

			continue;
		}
		if ((*ReadBuf == 1) && IsScanningClass)
		{
			fl = new CFileList;

			// Link on top (to make it easy)
			fl->m_Next = cl->m_FileList;
			cl->m_FileList = fl;

			MySscanf(ReadBuf,"\1%s\1 \1%s\1 %d %d \1%s\1 \1%s\1",
				fl->m_SrcName,
				fl->m_DstName,
				&fl->m_MenuMode,
				&fl->m_bPrimaryTarget,
				fl->m_MenuName,
				fl->m_UninstCmd);

			if (fl->m_bPrimaryTarget)
				LoadPrimaryTarget(fl->m_SrcName);

			if (!strcmp(fl->m_DstName,"(NULL)"))
				strcpy(fl->m_DstName, StripPath(fl->m_SrcName));
			else
			{
				if (fl->m_DstName[strlen(fl->m_DstName)-1] == '\\')
					strcat(fl->m_DstName,StripPath(fl->m_SrcName));
			}
		}
	}

	return TRUE;
}

void Setup::ExpandMacros(LPSTR Path)
{
	char tmp[512];
	tmp[0] = 0;
	LPCSTR p = Path;
	int Index = 0;

	while(*p)
	{
		if (*p == '<')
		{
			if (!strnicmp(p,"<Install Dir>", 13))
			{
				p += 13;
				strcpy(tmp, DestPath);
				if ((tmp[strlen(tmp) -1] != '\\') && (*p != '\\'))
					strcat(tmp, "\\");
			}
			else if (!strnicmp(p,"<Windows Dir>", 13))
			{
				p += 13;
				strcpy(tmp, WinDir);
				if ((tmp[strlen(tmp) -1] != '\\') && (*p != '\\'))
					strcat(tmp, "\\");
			}
			else if (!strnicmp(p,"<Windows System Dir>", 20))
			{
				p += 20;
				strcpy(tmp, SystemDir);
				if ((tmp[strlen(tmp) -1] != '\\') && (*p != '\\'))
					strcat(tmp, "\\");
			}

			Index = strlen(tmp);
		}
		else
		{
			tmp[Index++] += *p++;
			tmp[Index] = 0;
		}
	}

	strcpy(Path, tmp);
}

BOOL Setup::InstallFile(CClassList *cl, CFileList *fl)
{
	// Check if target exist.
	DWORD dwRval;
	static char szCurDir[MAX_PATH];
	static char szDestDir[MAX_PATH];
	static char szTmpFile[MAX_PATH];
	static char szCWD[MAX_PATH];
	UINT lpuCurDirLen = MAX_PATH;
	UINT lpuDestDirLen = MAX_PATH;
	UINT lpuTmpFileLen = MAX_PATH;
	int NumFound = 0;
	DWORD dwAttr;
	char cDestPath[MAX_PATH];
	char *pBuf = new char[1024 * 16];
	*pBuf = 0;
	char cPath[MAX_PATH];

	*szCurDir = 0;

	GetCurrentDirectory(MAX_PATH,szCWD);
	strcpy(cDestPath, cl->m_InstallDir);

	if ((cDestPath[strlen(cDestPath) -1] != '\\')
		&& (*cl->m_InstallDir != '\\'))
	{
		strcat(cDestPath, "\\");
	}

	strcat(cDestPath, fl->m_DstName);

	ExpandMacros(cDestPath);
	if (cDestPath[0] && (cDestPath[strlen(cDestPath) -1] == '\\'))
	{
		strcat(cDestPath, StripPath(fl->m_SrcName));
		strcpy(fl->m_DstName, cDestPath);	
	}

	// Prepere destignation path (if present)
	char *PrgPath = "";
	char *p = strrchr(cDestPath,'\\');
	if (p)
	{
		PrgPath = strdup(cDestPath);
		p = strrchr(PrgPath,'\\');
		*++p = 0;
	}

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"InstallFile: %s\n", PrgPath);

	// Make a quick scan for the file in Windows, system and system32
	if (cl->m_IsSystem)
	{
		// Prepere the error message in case we need it.
		sprintf(pBuf,"Warning. Multiple copies of the shared file %s is found in the following paths:",
			 StripPath(fl->m_DstName));

		// Try Windows
		sprintf(cPath,"%s\\%s", WinDir, StripPath(fl->m_DstName));
		if (GetFilePermissions(cPath, dwAttr))
		{
			strcpy(szCurDir,WinDir);
			strcat(pBuf,"\r\n"); 
			strcat(pBuf, cPath); 
			NumFound++;
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tTarget found in %s\n", WinDir);
		}

		// Try Windows\system
		sprintf(cPath,"%s\\System\\%s", WinDir, StripPath(fl->m_DstName));
		if (GetFilePermissions(cPath,dwAttr))
		{
			sprintf(szCurDir,"%s\\system",WinDir);
			strcat(pBuf, "\r\n"); 
			strcat(pBuf, cPath); 
			NumFound++;
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tTarget found in %s\\System\n", WinDir);
		}

		// Try Windows\system32
		sprintf(cPath,"%s\\System32\\%s", WinDir, StripPath(fl->m_DstName));
		if (GetFilePermissions(cPath,dwAttr))
		{
			sprintf(szCurDir,"%s\\system32",WinDir);
			strcat(pBuf, "\r\n"); 
			strcat(pBuf, cPath); 
			NumFound++;
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tTarget found in %s\\System32\n", WinDir);
		}

		if (NumFound > 1)
		{
			strcat(pBuf,"\r\nThis can cause problems with this and other programs. Please consider appropriate actions to resolve this possible problem.");
			MyMessageBox(pBuf);
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tUser alerted about %d targets...\n", NumFound);
		}
	}

	if (strchr(PrgPath,':'))
	{
		strcpy(pBuf, PrgPath);
		if (pBuf[strlen(pBuf) -1] == '\\')
			pBuf[strlen(pBuf) - 1] = 0;
	}
	else
		strcpy(pBuf,cl->m_IsSystem ? SystemDir : DestPath);

	if (*PrgPath && !strchr(PrgPath,':'))
	{
		if (pBuf[strlen(pBuf)-1] != '\\')
			strcat(pBuf, "\\");
		strcat(pBuf, PrgPath);
		if (pBuf[strlen(pBuf) -1] == '\\')
			pBuf[strlen(pBuf) - 1] = 0;
	}

	if (*PrgPath)
	{
		if (CreatePathForDirectory(pBuf))
		{
			fprintf(m_UnInstFp,"dir %s\n", pBuf);
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tDirectory %s created.\n", pBuf);
		}
	}

	{
		dwRval = 0;
		strcpy(szDestDir,pBuf);
		if (!*szCurDir)
			strcpy(szCurDir,pBuf);
	}

	sprintf(cPath,"%s\\%s",szDestDir, StripPath(fl->m_DstName));

	// Try to install the file
	dwRval = MyVerInstallFile(
		0,
		(char *)StripPath(fl->m_SrcName),
		(char *)StripPath(fl->m_DstName),
		szCWD,
		szDestDir,
		szCurDir,
		szTmpFile,
		&lpuTmpFileLen);

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tMyVerInstallFile() returned %d\n", dwRval);

	if (dwRval & VIF_SRCOLD)
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tTarget is newer than the install fille. Skipping...\n");
		delete pBuf;
		return FALSE;
	}

	if (dwRval & VIF_OUTOFSPACE)
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tVIF_OUTOFSPACE. Skipping...\n");
		MyMessageBox("The file can not be copied due to insufficient disk space.");
		delete pBuf;
		return FALSE;
	}

	if (dwRval)
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tTrying to force installation...\n");
		// Try to force the installation!
		// VIF_OUTOFSPACE
		dwRval = MyVerInstallFile(
		VIFF_FORCEINSTALL,
		(char *)StripPath(fl->m_SrcName),
		(char *)StripPath(fl->m_DstName),
		szCWD,
		szDestDir,
		szCurDir,
		szTmpFile,
		&lpuTmpFileLen);
	}
	
	// Somehow it reports out of memory under NT 351 when the file is in use
	if (dwRval & (VIF_FILEINUSE | VIF_OUTOFMEMORY))
	{
		// File is in use. We have to schedule the install for the next system reboot.
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tFile is in use... ");
		if ((OsType == OS_NT4) || (OsType == OS_NT351))
		{
			// Use NT MoveFileEx
			char TmpName[MAX_PATH *  2];
			sprintf(TmpName,"~%s", cPath);
			if (!CopyFile(StripPath(fl->m_SrcName), TmpName, FALSE))
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"NT: ***CopyFile(%s, %s) failed!\n",
					fl->m_SrcName, TmpName);
				MyMessageBox("MoveFileEx() failed.");
				return FALSE;
			}
			if (!MoveFileEx(TmpName, cPath, MOVEFILE_DELAY_UNTIL_REBOOT|MOVEFILE_REPLACE_EXISTING))
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"NT: ***MoveFileEx(%s, %s) failed!\n", 
					TmpName, cPath);
				MyMessageBox("MoveFileEx() failed.");
				return FALSE;
			}
			else
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"NT: MoveFileEx() OK\n");
				m_DoReboot = TRUE;
			}
		}
		else if (OsType == OS_WIN95)
		{
			strcpy(pBuf, cPath);
			strcat(pBuf, ".tmp");
			MoveFile(StripPath(fl->m_SrcName), pBuf);
			WritePrivateProfileString("rename", cPath, pBuf, "wininit.ini");
			m_DoReboot = TRUE;
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Win95: Scheduled replace at boot OK\n");
		}
		else
		{
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"*** Dont know how to install shared files on this system. Shared files will not be installed. if they are in use by Windows.\n");
			MyMessageBox("Dont know how to install shared files on this system. Shared files will not be installed. if they are in use by Windows.");
			delete pBuf;
			return FALSE;
		}
	}
	else if (dwRval)
	{
		sprintf(pBuf,"Failed to install file: %s\\%s --> %s\\%s\r\nFlags = ", 
			szCWD,
			StripPath(fl->m_SrcName),
			szDestDir,
			StripPath(fl->m_DstName));

		if (dwRval & VIF_TEMPFILE) strcat(pBuf, "VIF_TEMPFILE");
		if (dwRval & VIF_MISMATCH) strcat(pBuf, "VIF_MISMATCH");
		if (dwRval & VIF_SRCOLD) strcat(pBuf, "VIF_SRCOLD");
		if (dwRval & VIF_DIFFLANG) strcat(pBuf, "VIF_DIFFLANG");
		if (dwRval & VIF_DIFFCODEPG) strcat(pBuf, "VIF_DIFFCODEPG");
		if (dwRval & VIF_DIFFTYPE) strcat(pBuf, "VIF_DIFFTYPE");
		if (dwRval & VIF_WRITEPROT) strcat(pBuf, "VIF_WRITEPROT");
		if (dwRval & VIF_FILEINUSE) strcat(pBuf, "VIF_FILEINUSE");
		if (dwRval & VIF_OUTOFSPACE) strcat(pBuf, "VIF_OUTOFSPACE");
		if (dwRval & VIF_ACCESSVIOLATION) strcat(pBuf, "VIF_ACCESSVIOLATION");
		if (dwRval & VIF_SHARINGVIOLATION)strcat(pBuf, "VIF_SHARINGVIOLATION");
		if (dwRval & VIF_CANNOTCREATE) strcat(pBuf, "VIF_CANNOTCREATE");
		if (dwRval & VIF_CANNOTDELETECUR) strcat(pBuf, "VIF_CANNOTDELETECUR");
		if (dwRval & VIF_CANNOTRENAME) strcat(pBuf,  "VIF_CANNOTRENAME");
		if (dwRval & VIF_OUTOFMEMORY) strcat(pBuf, "VIF_OUTOFMEMORY");
		if (dwRval & VIF_CANNOTREADSRC) strcat(pBuf, "VIF_CANNOTREADSRC");
		if (dwRval & VIF_BUFFTOOSMALL) strcat(pBuf,  "VIF_BUFFTOOSMALL");
		if (dwRval & VIF_CANNOTREADDST) strcat(pBuf,  "VIF_CANNOTREADDST");
		MyMessageBox(pBuf);
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"%s\n", pBuf);
		delete pBuf;
		return FALSE;
	}

	if (((OsType == OS_NT4) || (OsType == OS_NT351)) && ((cl->m_IsShared || cl->m_IsSystem)))
	{
		// Increment Registry counters for shared files
		IncRegCnt(cPath);
	}

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\tFile is installed or scheduled OK\n");

	// Add log line
	fprintf(m_UnInstFp,"file %d %d %s\n", 
		cl->m_IsShared, cl->m_IsSystem, (LPCSTR)cPath);

	if (*fl->m_UninstCmd)
	{
		if (stricmp(fl->m_UninstCmd, "(NULL)"))
			fprintf(m_UnInstFp,"cmd %s\n", fl->m_UninstCmd);
	}

	if (fl->m_MenuMode)
	{
		if ((OsType == OS_NT4) || (OsType == OS_WIN95))
		{
			switch(fl->m_MenuMode)
			{		
				case 1: // User folder
					if (!UserProgramFolderDir[0] || !FolderName[0])
						goto skip;

					sprintf(pBuf, "%s\\%s", UserProgramFolderDir, FolderName);
					break;
				case 2: // System folders
					if (!ProgramFolderDir[0] || !FolderName[0])
						goto skip;

					sprintf(pBuf, "%s\\%s", ProgramFolderDir, FolderName);
					break;
				case 3: // Start group
					if (!StartUpMenu[0])
						goto skip;

					sprintf(pBuf, "%s", StartUpMenu);
					break;
			}
			if (!pBuf[0])
			{
				static BOOL HaveTold;
				if (HaveTold)
				{
					m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Warning: Setup can not create shortcut to the Start Menu. You have to add the application to the Start Menu manually...\n");
					MyMessageBox("Warning: Setup can not create shortcut to the Start Menu. You have to add the application to the Start Menu manually...");
					HaveTold = TRUE;
				}
				
				goto skip;
			}
			fprintf(m_UnInstFp,"dir %s\n", pBuf);
			CreateDirectory(pBuf,NULL);
			strcat(pBuf, "\\");
			strcat(pBuf, fl->m_MenuName);
			strcat(pBuf, ".lnk");
			CreateLink(cPath,pBuf,m_Name);
			fprintf(m_UnInstFp,"file 0 0 %s\n", (LPCSTR)pBuf);
		}
		else if (OsType == OS_NT351)
		{
			fprintf(m_UnInstFp,"pmngr %s@%s\n", (LPCSTR)FolderName, fl->m_MenuName);
			DDECreateGroup(FolderName);
			DDEAddProgramItem(fl->m_MenuName,cPath);
		}
	}
skip:
	delete pBuf;
	return TRUE;
}

LPCSTR StripPath(LPCSTR Path)
{
	LPCSTR Name;

	if ((Name = strrchr(Path,'\\')) != NULL)
		++Name;
	else
		Name = Path;

	return Name;
}


// CreateLink - uses the shell's IShellLink and IPersistFile interfaces 
//   to create and store a shortcut to the specified object. 
// Returns the result of calling the member functions of the interfaces. 
// lpszPathObj - address of a buffer containing the path of the object 
// lpszPathLink - address of a buffer containing the path where the 
//   shell link is to be stored 
// lpszDesc - address of a buffer containing the description of the 
//   shell link 
 
HRESULT Setup::CreateLink(LPCSTR lpszPathObj, LPCSTR lpszPathLink, LPCSTR lpszDesc) 
{ 
    HRESULT hres; 
    IShellLink* psl; 
		LPSTR WorkingDir = strdup(lpszPathObj);
		LPSTR p = strrchr(WorkingDir,'\\');
		if (p && *p)
			*p = 0;
 
		CoInitialize(NULL);
    // Get a pointer to the IShellLink interface. 
    hres = CoCreateInstance(CLSID_ShellLink, NULL, 
        CLSCTX_INPROC_SERVER, IID_IShellLink, (void **)&psl); 
    if (SUCCEEDED(hres)) { 
        IPersistFile* ppf; 
 
        // Set the path to the shortcut target, and add the 
        // description. 
        psl->SetPath(/*psl,*/ lpszPathObj); 
        psl->SetDescription(/*psl,*/ lpszDesc); 
				psl->SetWorkingDirectory(WorkingDir);
 
       // Query IShellLink for the IPersistFile interface for saving the 
       // shortcut in persistent storage. 
        hres = psl->QueryInterface(/*psl, */IID_IPersistFile, 
            (void **)&ppf); 
 
        if (SUCCEEDED(hres)) { 
            WORD wsz[MAX_PATH]; 
 
            // Ensure that the string is ANSI. 
            MultiByteToWideChar(CP_ACP, 0, lpszPathLink, -1, 
                wsz, MAX_PATH); 
 
            // Save the link by calling IPersistFile::Save. 
            hres = ppf->Save(/*ppf,*/ wsz, TRUE); 
            ppf->Release(/*ppf*/); 
        } 
        psl->Release(/*psl*/); 
    } 
		CoUninitialize();
    return hres; 
} 
 

BOOL Setup::IncRegCnt(LPCSTR Path)
{
	DWORD dwNihl, dwSize = sizeof(DWORD);
	LONG Rval;
	HKEY hkey;
	DWORD dwCounter = 1;
	
	if ((Rval = RegOpenKeyEx(
		HKEY_LOCAL_MACHINE,
		"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\SharedDLLs",
		0,
		KEY_QUERY_VALUE | KEY_WRITE,
		&hkey)) == ERROR_SUCCESS) 
	{
		if ((Rval = RegQueryValueEx(
			hkey,
			Path,
			NULL,
			&dwNihl,
			(LPBYTE)&dwCounter,
			&dwSize)) == ERROR_SUCCESS)
		{
			++dwCounter;
		}
		if (RegSetValueEx(
			hkey,
			Path,
			0,
			REG_DWORD,
			(LPBYTE)&dwCounter,
			dwSize) != ERROR_SUCCESS)
		{
			MyMessageBox("Failed to increment the registery counter for this shared file.");
		}

		RegCloseKey(hkey);
		fprintf(m_UnInstFp,"inc %s\n", Path);
		return TRUE;
	}
	else
	{
		char pBuf[512];
		sprintf(pBuf,"Failed to open the registery counter for this shared file. Error = %u",
			Rval);
		MyMessageBox(pBuf);
	}

	return FALSE;
}

// Have to do this job by ourself since
// the library function dont work with 
// long filenames under Win95 and God knows what else...
DWORD Setup::MyVerInstallFile(DWORD uFlags,LPTSTR szSrcFileName,
	LPTSTR szDestFileName,LPTSTR szSrcDir,LPTSTR szDestDir,
	LPTSTR szCurDir, LPTSTR szTmpFile,PUINT lpuTmpFileLen)
{

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t  MyVerInstallFile(%s --> %s)\n", szSrcFileName, szDestFileName);
	BOOL HaveBoth = FALSE;
	// We don't use temporary files :-)
	szTmpFile = "";
	*lpuTmpFileLen = 0;

	if (!strchr(szCurDir,'\\'))
		*szCurDir = 0;

	// Make the full paths
	char cSrcPath[MAX_PATH], cDestPath[MAX_PATH];

	sprintf(cSrcPath,"%s\\%s", szSrcDir, szSrcFileName);
	sprintf(cDestPath,"%s\\%s", szCurDir, szDestFileName);

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tSrc = %s\n", cSrcPath);
	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tDst = %s\n", cDestPath);

	// Start with a simple check to see if both files exist
	HANDLE hSrc, hDest;
	WIN32_FIND_DATA dSrc, dDest;
	
	if ((hSrc = FindFirstFile(cSrcPath, &dSrc)) == INVALID_HANDLE_VALUE)
	{
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\t**VIF_CANNOTREADSRC\n");
		return VIF_CANNOTREADSRC;
	}
	FindClose(hSrc);
	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tSrc exist.\n");

	if (*szCurDir && !(uFlags & VIFF_FORCEINSTALL))
	{
		// We must check if the destination file is newer than the source file
		if ((hDest = FindFirstFile(cDestPath, &dDest)) != INVALID_HANDLE_VALUE)
		{
			// Both files exist.
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tDst exist\n");
			FindClose(hDest);
			HaveBoth = TRUE;

			// Compare verison
			DWORD BufferSize, dwDummy;
			UINT Dummy;
			char *Vbuf = NULL;
			LPVOID vPtr;
			VS_FIXEDFILEINFO vSrc, vDest;

			BufferSize = GetFileVersionInfoSize((char *)(LPCSTR)cSrcPath,&dwDummy);
			if (!BufferSize)
			{
				Dummy = GetLastError();
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tFailed to get version info size on src. Error %d\n", Dummy); 
				switch(Dummy)
				{
					case ERROR_FILE_NOT_FOUND:
					case ERROR_PATH_NOT_FOUND:
					case ERROR_ACCESS_DENIED:
					case ERROR_NO_MORE_FILES:
					case ERROR_TOO_MANY_OPEN_FILES:
					case ERROR_OUTOFMEMORY:
					case ERROR_SHARING_VIOLATION:
					case ERROR_NETWORK_BUSY:
						return VIF_CANNOTREADSRC;
					default:
						goto datecmp; // We assume that the file dont have a version
				}
			} 
				
			Vbuf = new char[BufferSize];

			if (!GetFileVersionInfo((char *)(LPCSTR)cSrcPath,0,BufferSize,(LPVOID)Vbuf))
			{
				//return VIF_CANNOTREADSRC;
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tFailed to get version information on src.\n"); 
				goto datecmp;
			}

			if (!VerQueryValue(Vbuf,TEXT("\\"), &vPtr, &Dummy))
			{
				//return VIF_CANNOTREADSRC;
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tsrc: VerQueryValue() failed.\n"); 
				goto datecmp;
			}

			ASSERT(BufferSize >= sizeof(VS_FIXEDFILEINFO));
			ASSERT(Dummy >= sizeof(VS_FIXEDFILEINFO));

			memcpy(&vSrc,vPtr,sizeof(VS_FIXEDFILEINFO));
			delete Vbuf;

				// Get the destination info.
			BufferSize = GetFileVersionInfoSize((char *)(LPCSTR)cDestPath,&dwDummy);
		
			if (!BufferSize)
			{
				Dummy = GetLastError();
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tFailed to get version info size on dst. Error %d\n", Dummy); 
				switch(Dummy)
				{
					case ERROR_FILE_NOT_FOUND:
					case ERROR_PATH_NOT_FOUND:
					case ERROR_ACCESS_DENIED:
					case ERROR_NO_MORE_FILES:
					case ERROR_TOO_MANY_OPEN_FILES:
					case ERROR_OUTOFMEMORY:
					case ERROR_SHARING_VIOLATION:
					case ERROR_NETWORK_BUSY:
						return VIF_CANNOTREADDST;
					default:
						goto copy; // We assume that the file dont have a version
				}
			}
		
			Vbuf = new char[BufferSize];

			if (!GetFileVersionInfo((char *)(LPCSTR)cDestPath,0,BufferSize,(LPVOID)Vbuf))
			{
				//return VIF_CANNOTREADSRC;
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tFailed to get version information on src.\n"); 
				goto datecmp;
			}

			if (!VerQueryValue(Vbuf,TEXT("\\"), &vPtr, &Dummy))
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tdst: VerQueryValue() failed.\n"); 
				//return VIF_CANNOTREADSRC;
				goto datecmp;
			}

			ASSERT(BufferSize >= sizeof(VS_FIXEDFILEINFO));
			ASSERT(Dummy >= sizeof(VS_FIXEDFILEINFO));
			memcpy(&vDest,vPtr,sizeof(VS_FIXEDFILEINFO));
	
			delete Vbuf;

			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tComparsion are done by file version.\n");
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tsrc=(%u,%u) dst=(%u,%u).\n",
				vSrc.dwFileVersionMS, vSrc.dwFileVersionLS,
				vDest.dwFileVersionMS, vDest.dwFileVersionLS );
	
			// Lets compare the version numbers
			if ((vDest.dwFileVersionMS > vSrc.dwFileVersionMS) 
				|| (vDest.dwFileVersionMS == vSrc.dwFileVersionMS) && (vDest.dwFileVersionLS >= vSrc.dwFileVersionLS))
			//if ((vDest.dwFileVersionMS > vSrc.dwFileVersionMS) 
			//	|| (vDest.dwFileVersionMS == vSrc.dwFileVersionMS) && (vDest.dwFileVersionLS > vSrc.dwFileVersionLS))
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tDst is equal or more up to date than src.\n");
				return VIF_SRCOLD | VIF_MISMATCH;
			}

			// If we get this far, we *ignore* the file date..
			goto copy;

datecmp:
			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tComparsion are defaulting to file date...\n");
			// Compare file dates
			if (HaveBoth && CompareFileTime(&dSrc.ftLastWriteTime,&dDest.ftLastWriteTime) <= 0)
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tDst are newer than the install file.\n");
				return VIF_SRCOLD;
			}

			// TODO: Add comparsion on language, OS and codepage.
		}
	}
copy:

	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tPreparing for copy...\n");

	if (*szCurDir && !(uFlags & VIFF_DONTDELETEOLD))
	{
		// Delete the old file if it is read-only
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tOld file must (eventually) be deleted... ");
		DWORD dwAttr = 0;
		if (GetFilePermissions(cDestPath,dwAttr) && (dwAttr & FILE_ATTRIBUTE_READONLY))
		{
			if (!(uFlags & VIFF_FORCEINSTALL))
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Not called with permission to delete readonly files... Aborting operation.\n");
				return VIF_WRITEPROT;
			}
			
			DWORD dwAttributes = GetFileAttributes(cDestPath);
			dwAttributes &= ~FILE_ATTRIBUTE_READONLY;
			SetFileAttributes(cDestPath, dwAttributes);

			m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"File is prepered for delete... ");
		}
		if (!DeleteFile(cDestPath))
		{
			DWORD Err = GetLastError();
			if (Err != ERROR_FILE_NOT_FOUND)
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp," **Failed. Error code %d\n", Err);
				if (dwAttr & FILE_ATTRIBUTE_SYSTEM)
				{
					m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tSystem attribute on file.. Assuming file in use...\n");
					return VIF_FILEINUSE;
				}

				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\t**Aborting operation du to error.\n");
				switch(Err)
				{
					case ERROR_ACCESS_DENIED:
						return VIF_FILEINUSE | VIF_CANNOTDELETE;
				
					case ERROR_PATH_NOT_FOUND:
					case ERROR_NO_MORE_FILES:
					case ERROR_TOO_MANY_OPEN_FILES:
					case ERROR_OUTOFMEMORY:
					case ERROR_SHARING_VIOLATION:
					case ERROR_NETWORK_BUSY:
						return VIF_CANNOTDELETE;
				}
			}
		}
	}


	sprintf(cDestPath,"%s\\%s", szDestDir, szDestFileName);
	// Lets do the transfere.

	if (!CopyFile(cSrcPath,cDestPath,FALSE))
	{
		char *pBuf = new char[1024 * 16];
		sprintf(pBuf,"%s --> %s\r\nCopy failed.", cSrcPath, cDestPath);
		MyMessageBox(pBuf);
		DWORD Err = GetLastError();
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tpBuf Error code %d\n", pBuf, Err);
		delete pBuf;
		switch(Err)
		{
			case ERROR_FILE_NOT_FOUND:
				return VIF_CANNOTREADSRC;
			case ERROR_SHARING_VIOLATION:
				return VIF_SHARINGVIOLATION | VIF_FILEINUSE;
			case ERROR_ACCESS_DENIED:
				return VIF_ACCESSVIOLATION;
			default:
				return VIF_CANNOTCREATE;
		}
	}
	m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\t\tFile installed OK.\n");
	return 0;
}


BOOL Setup::GetFilePermissions(LPCSTR Path, DWORD& Perms)
{
	Perms = GetFileAttributes(Path);
	
	return Perms != 0xFFFFFFFF;
}


// Create a directory, and any nodes down the way...
BOOL Setup::CreatePathForDirectory(LPCSTR Path)
{
	char *MyPath = strdup(Path);
	char *p = MyPath;
	BOOL Rval = FALSE;
	int psave;

	while(*p)
	{
		while(*p && *p != '\\')
			++p;

		psave = *p;
		if (*p == '\\')
		{
			*p = 0;
		}

		if (strlen(MyPath) > 3)
			Rval = CreateDirectory(MyPath, NULL);

		*p = psave;
		if (*p)
			++p;
	}

	delete MyPath;
	return Rval;
}


void Setup::MyMessageBox(LPCSTR Text)
{
	MessageBox(NULL, Text, "War Setup - ERROR", MB_OK);
}


BOOL Setup::Get(LPSTR Buf,int Bytes, FILE *fp)
{
	if (!fgets(Buf,Bytes,fp))
		return FALSE;

	int Len = strlen(Buf);
	LPSTR p = Buf + (Len - 1);
	while(Len)
	{
		if(p >= Buf)
		{
			if(strchr("\r\n\t ", *p))
			{
				*p = 0;
				--Len;
			}
			else
				break;
		}
		--p;
	}
	
	return TRUE;
}
	

void Setup::LoadPrimaryTarget(LPCSTR Path)
{
	LPCSTR Name = strrchr(Path, '\\');
	if (Name)
		++Name;
	else
		Name = Path;

	strcpy(m_PrimaryTarget, Name);

	DWORD BufferSize, dwDummy;
	UINT Dummy;
	char *Vbuf = NULL;
	VS_FIXEDFILEINFO *v;

	BufferSize = GetFileVersionInfoSize((char *)(LPCSTR)Name,&dwDummy);
	if (!BufferSize)
		return;

	Vbuf = new char[BufferSize];

	if (!GetFileVersionInfo((char *)(LPCSTR)Name,0,BufferSize,(LPVOID)Vbuf))
		return;

	if (!VerQueryValue(Vbuf,TEXT("\\"), (LPVOID *)&v, &Dummy))
		return;

	sprintf(m_Version,"%u.%u.%u.%u", 
		HIWORD(v->dwProductVersionMS),
		LOWORD(v->dwProductVersionMS),
		HIWORD(v->dwProductVersionLS),
		LOWORD(v->dwProductVersionLS));

	MyGetVerInfo(Vbuf, (LPSTR *)&m_Version, "ProductVersion");
	MyGetVerInfo(Vbuf, (LPSTR *)&m_Copyright, "LegalCopyright");
	MyGetVerInfo(Vbuf, (LPSTR *)&m_VerboseName, "ProductName");
	MyGetVerInfo(Vbuf, (LPSTR *)&m_CompanyName, "CompanyName");
	MyGetVerInfo(Vbuf, (LPSTR *)&m_Comments, "Comments");
	MyGetVerInfo(Vbuf, (LPSTR *)&m_SpecialBuild, "SpecialBuild");
	MyGetVerInfo(Vbuf, (LPSTR *)&m_LegalTrademarks, "LegalTrademarks");
	delete Vbuf;
}

BOOL Setup::MyGetVerInfo(LPVOID Vbuf, LPSTR *Val, LPCSTR Name)
{
	char Key[128];
	UINT  uiVerSize ;   
	UINT  uiSize ;   
	DWORD* lpBuffer;

	if (VerQueryValue(Vbuf,"\\VarFileInfo\\Translation",(
		void**)&lpBuffer, &uiVerSize))
	{
		if (uiVerSize)
		{
			sprintf(Key, "\\StringFileInfo\\%04hX%04hX\\%s",
				LOWORD(*lpBuffer), HIWORD(*lpBuffer), Name);
	
			LPSTR p;
			if (VerQueryValue(Vbuf, Key, (void **)&p, &uiSize))
			{
				if (p)
				{
					strcpy((LPSTR)Val, p);
					return TRUE;
				}
			}
		}
	}

	return FALSE;
}

BOOL Setup::QueryFileInfoForListCntr(LPCSTR Name, HWND hWnd, int Row, int Col, ...)
{
	DWORD BufferSize, dwDummy;
	char *Vbuf = NULL;

	BufferSize = GetFileVersionInfoSize((char *)(LPCSTR)Name,&dwDummy);
	if (!BufferSize)
		return FALSE;

	Vbuf = new char[BufferSize];

	if (!GetFileVersionInfo((char *)(LPCSTR)Name,0,BufferSize,(LPVOID)Vbuf))
		return FALSE;

	char Key[128];
	UINT  uiVerSize ;   
	UINT  uiSize ;   
	DWORD* lpBuffer;
	va_list ap;
	va_start(ap, Col);
	LPCSTR InfoName;

	while(InfoName = va_arg(ap, LPCSTR))
	{
		if (VerQueryValue(Vbuf,"\\VarFileInfo\\Translation",(
			void**)&lpBuffer, &uiVerSize))
		{
			if (uiVerSize)
			{
				sprintf(Key, "\\StringFileInfo\\%04hX%04hX\\%s",
					LOWORD(*lpBuffer), HIWORD(*lpBuffer), InfoName);
	
				LPSTR p;
				if (VerQueryValue(Vbuf, Key, (void **)&p, &uiSize))
				{
					if (p)
					{	
						LVITEM li;
						memset(&li, 0, sizeof(li));
						li.mask = LVIF_TEXT;
						li.iItem = Row;
						li.iSubItem = Col;
						li.pszText = p;
						li.cchTextMax =	strlen(li.pszText);
						SendMessage(hWnd, LVM_SETITEM, 0, (LPARAM)&li);
					}
				}
			}
		}
		++Col;
	}
	delete Vbuf;
	return TRUE;
}

int GetPercent(int CurrFile, int NumFiles);
int GetPercent(int CurrFile, int NumFiles)
{
	int Percent = (int)(((float)CurrFile / (float)NumFiles) * (float)100);
	return Percent;
}

BOOL Setup::InstallSelection(HWND hWndProgress, HWND hWndClass, int Mode, int Mask)
{
		int NumFiles = 0, CurrFile = 0;
		CFileList *fl;
		CClassList *cl;
		int Index;
		int Percent = 0;
		char cBuf[MAX_PATH * 2];
		char cTmp[MAX_PATH * 2];

		// Count the files we will copy
		for(cl = m_FileClassRoot, Index = 0; cl; cl = cl->m_Next, Index++)
		{
			switch(Mode)
			{
				case 0: // Normal
					if (!cl->m_IsTypical)
					{
						cl->m_Include = FALSE;
						continue;
					}
					cl->m_Include = TRUE;
					break;

				case 1: // Minimum
					if (!cl->m_IsMinimum)
					{
						cl->m_Include = FALSE;
						continue;
					}
					cl->m_Include = TRUE;
					break;

				case 2: // Custom
					if (Index < 10)
					{
						if (!(Mask & (1 << Index)))
						{
							cl->m_Include = FALSE;
							continue;
						}
					}
					cl->m_Include = TRUE;
					break;
			}

			for(fl = cl->m_FileList; fl; fl = fl->m_Next)
			{
				++NumFiles;				
			}
		}

		// We are ready to start copying files.
		// Open the log file that Uninstall will use
		if (!CreateDirectory(DestPath,NULL))
		{
			if (GetLastError() != ERROR_ALREADY_EXISTS)
			{
				MyMessageBox("Failed to create destination directory. Install aborted.");
				return FALSE;
			}
		}

		sprintf(cBuf, "%s\\.Install.Log", DestPath);

		if ((m_LogFp = fopen(cBuf,"w")) == NULL)
		{
			MyMessageBox("Failed to create Log file. Install aborted.");
			return FALSE;
		}

		sprintf(cBuf, "%s\\.UnInst.inf", DestPath);
	
		// Open uninstall log file. If the file exist, we assume upgrade and
		// append to the end of it. That way uninstall should work also after
		// an upgrade.
		if ((m_UnInstFp = fopen(cBuf,"a+")) == NULL)
		{
			MyMessageBox("Failed to create Uninstall information file. Install aborted.");
			return FALSE;
		}

		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Ready to install %s to %s\n", m_Name, DestPath);
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Windows dir = '%s'\nWindows system dir = '%s'\n",
			WinDir, SystemDir);

		fseek(m_UnInstFp,0L,SEEK_END);
		if (!ftell(m_UnInstFp))
		{
			fprintf(m_UnInstFp,"%s\n", (LPCSTR)m_Name);
			fprintf(m_UnInstFp,"dir %s\n", DestPath);
		}

		// Start the copy process

		for(cl = m_FileClassRoot, Index = 0; cl; cl = cl->m_Next, Index++)
		{
			if (!cl->m_Include)
			{
				m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"Skipping %s...\n", cl->m_Name);
				continue;
			}

			sprintf(cBuf, "Copying files (%s)...", cl->m_Name);
			SetWindowText(hWndClass, cBuf);
			PumpMessages();

			for(fl = cl->m_FileList; fl; fl = fl->m_Next)
			{
				UpdateProgress(hWndProgress, fl->m_DstName, GetPercent(CurrFile++, NumFiles));
				PumpMessages();
				InstallFile(cl,fl);
				PumpMessages();
			}
		}

		UpdateProgress(hWndProgress, "",100);
		SetWindowText(hWndClass, "Registering Uninstall program...");
		PumpMessages();

		sprintf(cBuf, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\%s",
			m_Name);

		CreateRegisteryKey(HKEY_LOCAL_MACHINE,cBuf);

		sprintf(cBuf, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\%s\\DisplayName",
			m_Name);
		SetRegisteryString(HKEY_LOCAL_MACHINE,cBuf,m_Name,strlen(m_Name));

		sprintf(cBuf, "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall\\%s\\UninstallString",
			m_Name);
		sprintf(cTmp, "%s\\Uninstall.exe \"%s\\.UnInst.inf\"", DestPath, DestPath);
		SetRegisteryString(HKEY_LOCAL_MACHINE,cBuf,cTmp,strlen(cTmp));

		fclose(m_UnInstFp);
		m_LogFp == NULL ? 0 :  fprintf(m_LogFp,"\nInstall done.%s\n", m_DoReboot ? " Machine must reboot" : "");
		fclose(m_LogFp);

	return TRUE;
}

CFileList::CFileList()
{
	m_SrcName[0] = 0;
	m_DstName[0] = 0;
	m_MenuMode = 0;
	m_MenuName[0] = 0;
	m_UninstCmd[0] = 0;
	m_bPrimaryTarget = FALSE;
	m_Next = NULL;
}

