
// Extension dll for War Install 2.0
// Customized support for the War FTP Daemon 1.70 installation process
// Copyright 1998 by Jarle (jgaa) Aase
// All rights reserved.
//
// This source code can be used as an example on how to
// do a plugin for War Install 2.0

#define WAR_SETUP_DLL

#include <windows.h>
#include <string.h>
#include <stdio.h>
#include "..\WarSetup\WarSetup.h"
#include "..\WarSetup\SetupFiles.h"
#include "..\WarSetup\Dialogs.h"
#include "shlobj.h"
#include <ODBCInst.H>

#include "Ward170.h" // Plugin dll header for this plugin
#include ".\resource.h"

void WritePrivateProfileInt(LPCSTR Hdr, LPCSTR Tag, int val, LPCSTR File);
BOOL MyStartService(LPCSTR Name);
BOOL ControlService(LPCSTR Name, DWORD Command, DWORD *pStatus);
void ShutDownServers(Setup *pSetup);
BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam);

HINSTANCE DllhInst;

// Local prototypes
BOOL WINAPI DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved);
BOOL WINAPI DllMain (HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	DllhInst = (HINSTANCE)hInst;
	return TRUE;
}

// Export function for WarInstall initialization
// Called after the default pages are set up, but before the screen is updated
// Return FALSE to abort the setup program
BOOL WINAPI InitPlugin(
	HWND hWnd, // WarSetup main window
	Setup *pSetup, // Setup processor
	PropertyPageList *pPropertySheet) // The property sheet, including default pages, except for the finish page
{
	// Set up our own property pages.
	// NB: Use DllhInst to avoid conflicts with the resoures in Setup.exe!!

	{	
		WelcomeWard170Page *p = new WelcomeWard170Page(pSetup);
		if (p->Create(IDD_SELECT_ROOT, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		SetupModeWard170Page *p = new SetupModeWard170Page(pSetup);
		if (p->Create(IDD_WAR170_INSTALLMODE, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}
	
	{	
		UpgradePathWard170Page *p = new UpgradePathWard170Page(pSetup);
		if (p->Create(IDD_WARD170_UPGRADE, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		SystemNameWard170Page *p = new SystemNameWard170Page(pSetup);
		if (p->Create(IDD_WARD170_SYSTEMNAME, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		ServiceWard170Page *p = new ServiceWard170Page(pSetup);
		if (p->Create(IDD_WARD170_SERVICE, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		NameTagWard170Page *p = new NameTagWard170Page(pSetup);
		if (p->Create(IDD_WARD170_TAG, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		RootFileWard170Page *p = new RootFileWard170Page(pSetup);
		if (p->Create(IDD_WARD170_NEWROOT, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		SecurityWard170Page *p = new SecurityWard170Page(pSetup);
		if (p->Create(IDD_WARD170_OPTIONS, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		PasswordWard170Page *p = new PasswordWard170Page(pSetup);
		if (p->Create(IDD_WARD170_PASSWD, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		EmailWard170Page *p = new EmailWard170Page(pSetup);
		if (p->Create(IDD_WARD170_EMAIL, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		FTPPortWard170Page *p = new FTPPortWard170Page(pSetup);
		if (p->Create(IDD_WARD170_FTPPORT, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		ReadyToGoWard170Page *p = new ReadyToGoWard170Page(pSetup);
		if (p->Create(IDD_WARD170_PRE_PROCESS, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	{	
		ProcessWard170Page *p = new ProcessWard170Page(pSetup);
		if (p->Create(IDD_PROCESS, hWnd, DllhInst))
			pPropertySheet->AddPage(p);
	}

	return TRUE;
}

BOOL CALLBACK InfoDlgProc(HWND hwndDlg, UINT uMsg,WPARAM wParam,LPARAM lParam);
BOOL CALLBACK InfoDlgProc(HWND hwndDlg, UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_INITDIALOG:
		case WM_CREATE:
			return TRUE;
	}
	return FALSE;
}

// Export function for WarInstall initialization
// Called after all property pares are set up, but before the screen is updated
// Return FALSE to abort the setup program
BOOL WINAPI PreperePlugin(
	HWND hWnd, // WarSetup main window
	Setup *pSetup, // Setup processor
	PropertyPageList *pPropertySheet) // The property sheet, including default pages, except for the finish page
{
	HWND hDlg;
	if ((hDlg = CreateDialog(DllhInst, MAKEINTRESOURCE(IDD_WAITFORSHTD),
		hWnd, (DLGPROC)InfoDlgProc)) != NULL)
	{
		UpdateWindow(hDlg);
		ShowWindow(hDlg, SW_SHOW);
	}
	ShutDownServers(pSetup);
	if (hDlg)
	{
		CloseWindow(hDlg);
		DestroyWindow(hDlg);
	}
	return TRUE;
}

// Export function for WarInstall initialization
// Called just after the first property-page is displayed
// Return FALSE to abort the setup program
BOOL WINAPI PreProcessPlugin(
	HWND hWnd, // WarSetup main window
	Setup *pSetup, // Setup processor
	PropertyPageList *pPropertySheet) // The property sheet, including default pages, except for the finish page
{
	return TRUE;
}

// Export function for WarInstall initialization
// Called before the setup thread exits
// Return value is ignored
BOOL WINAPI PostProcessPlugin(
	HWND hWnd, // WarSetup main window
	Setup *pSetup, // Setup processor
	PropertyPageList *pPropertySheet) // The property sheet, including default pages, except for the finish page
{
	ShellExecute(hWnd, "open","default.wdm", NULL, NULL, SW_SHOW);
	return TRUE;
}

// Export function for WarInstall initialization
// Called if the user press the Cancel button
// Return FALSE to deny the cancel operation
BOOL WINAPI OnCancelPlugin(
	HWND hWnd, // WarSetup main window
	Setup *pSetup, // Setup processor
	PropertyPageList *pPropertySheet) // The property sheet, including default pages, except for the finish page
{
	return TRUE;
}



//////////////////////////////////////////////////////////////////////////
// WelcomeWard170Page
//////////////////////////////////////////////////////////////////////////

WelcomeWard170Page::WelcomeWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "WelcomeWard170Page";
	m_IsOK = TRUE;
}

BOOL WelcomeWard170Page::OnActivate()
{
	PropertyPage::OnActivate();
	m_pOrigin->SetAsFirst("WelcomeWard170Page");
	m_pOrigin->EnableCancelButton(FALSE);
	m_pOrigin->EnablePrevButton(FALSE);

	{
		InstallModePage *p = (InstallModePage *)m_pOrigin->GetPage("InstallModePage");
		SetupModeWard170Page *pSM = (SetupModeWard170Page *)m_pOrigin->GetPage("SetupModeWard170Page");
		
		if (p->m_Mode == InstallModePage::mode_custom)
		{
			InstallCustomPage *p = (InstallCustomPage *)m_pOrigin->GetPage("InstallCustomPage");
			for(int i =0; i < InstallCustomPage::NUM_MODULES; i++)
			{
				if (p->m_Data[i].m_pClass)
				{
					if (!stricmp(p->m_Data[i].m_pClass->m_Name, "FTP Server"))
						pSM->m_bDoServerInstall = p->m_Data[i].m_bSelected;
					if (!stricmp(p->m_Data[i].m_pClass->m_Name, "Manager"))
						pSM->m_bDoClientInstall = p->m_Data[i].m_bSelected;
				}
			}
		}

		if (!pSM->m_bDoServerInstall)
		{
			m_pOrigin->Enable("UpgradePathWard170Page", FALSE);
			m_pOrigin->Enable("NameTagWard170Page", FALSE);
			m_pOrigin->Enable("ServiceWard170Page", FALSE);
			m_pOrigin->Enable("RootFileWard170Page", FALSE);
			m_pOrigin->Enable("FTPPortWard170Page", FALSE);
			m_pOrigin->Enable("SystemNameWard170Page", FALSE);
			m_pOrigin->Enable("SecurityWard170Page", FALSE);
			m_pOrigin->Enable("PasswordWard170Page", FALSE);
			m_pOrigin->Enable("EmailWard170Page", FALSE);
			m_pOrigin->Enable("NameTagWard170Page", FALSE);
			m_pOrigin->Enable("ServiceWard170Page", FALSE);
			m_pOrigin->Enable("FTPPortWard170Page", FALSE);
			m_pOrigin->Enable("SetupModeWard170Page", FALSE);
		}
	}

	return TRUE;
}


//////////////////////////////////////////////////////////////////////////
// SetupModeWard170Page
//////////////////////////////////////////////////////////////////////////

SetupModeWard170Page::SetupModeWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "SetupModeWard170Page";
	m_IsOK = TRUE;
	m_Mode = mode_new;
	if (FindWindow("WarTrayIconClass", "WarTrayIcon"))
		m_Mode = mode_upgr_170;
	
	m_bDoServerInstall = TRUE;
	m_bDoClientInstall = TRUE;
}

BOOL SetupModeWard170Page::OnActivate()
{
	PropertyPage::OnActivate();
	return TRUE;
}

BOOL SetupModeWard170Page::OnProcess()
{
	HWND wTray = NULL;
	
	return TRUE;
}

void SetupModeWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		for(int i = 0; i < (int)mode_invalid; i++)
		{
			HWND hw = GetDlgItem(m_hWnd, IDC_RADIO1 + i);
			if (SendMessage(hw, BM_GETCHECK, 0, 0) == BST_CHECKED)
			{
				m_Mode = (SetupModesDef) i;
				break;
			}
		}	
	}
	else
	{
		for(int i = 0; i < (int)mode_invalid; i++)
		{
			HWND hw = GetDlgItem(m_hWnd, IDC_RADIO1 + i);
			SendMessage(hw, BM_SETCHECK, (m_Mode == (SetupModesDef) i) 
				? BST_CHECKED : BST_UNCHECKED, 0);
		}	
	}

	PropertyPage::OnUpdate();
}

BOOL SetupModeWard170Page::OnNext()
{
	// Set up the appropriate pages for the install mode
	BOOL bSetTagPage = FALSE;

	{
		InstallModePage *p = (InstallModePage *)m_pOrigin->GetPage("InstallModePage");
		if (p->m_Mode == InstallModePage::mode_custom)
		{
			InstallCustomPage *p = (InstallCustomPage *)m_pOrigin->GetPage("InstallCustomPage");
			for(int i =0; i < InstallCustomPage::NUM_MODULES; i++)
			{
				if (p->m_Data[i].m_pClass)
				{
					if (!stricmp(p->m_Data[i].m_pClass->m_Name, "FTP Server"))
						m_bDoServerInstall = p->m_Data[i].m_bSelected;
					if (!stricmp(p->m_Data[i].m_pClass->m_Name, "Manager"))
						m_bDoClientInstall = p->m_Data[i].m_bSelected;
				}
			}
		}
	}

	switch(m_Mode)
	{
		case mode_new:
			m_pOrigin->Enable("UpgradePathWard170Page", FALSE);
			m_pOrigin->Enable("NameTagWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("ServiceWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("RootFileWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("FTPPortWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("SystemNameWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("SecurityWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("PasswordWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("EmailWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("NameTagWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("ServiceWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("FTPPortWard170Page", m_bDoServerInstall);
			break;
		case mode_upgr_pre170:
			m_pOrigin->Enable("UpgradePathWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("NameTagWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("ServiceWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("RootFileWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("FTPPortWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("SecurityWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("PasswordWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("EmailWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("NameTagWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("ServiceWard170Page", m_bDoServerInstall);
			m_pOrigin->Enable("FTPPortWard170Page", m_bDoServerInstall);
			break;
		case mode_upgr_170:
			m_pOrigin->Enable("UpgradePathWard170Page", m_bDoServerInstall);
			bSetTagPage = m_bDoServerInstall; // Use old tag value
			m_pOrigin->Enable("NameTagWard170Page", FALSE);
			m_pOrigin->Enable("ServiceWard170Page", FALSE);
			m_pOrigin->Enable("RootFileWard170Page", FALSE);
			m_pOrigin->Enable("FTPPortWard170Page", FALSE);
			m_pOrigin->Enable("SystemNameWard170Page", FALSE);
			m_pOrigin->Enable("SecurityWard170Page", FALSE);
			m_pOrigin->Enable("PasswordWard170Page", FALSE);
			m_pOrigin->Enable("EmailWard170Page", FALSE);
			m_pOrigin->Enable("NameTagWard170Page", FALSE);
			m_pOrigin->Enable("ServiceWard170Page", FALSE);
			m_pOrigin->Enable("FTPPortWard170Page", FALSE);
			break;
	}

	return PropertyPage::OnNext();
}


//////////////////////////////////////////////////////////////////////////
// RootFileWard170Page
//////////////////////////////////////////////////////////////////////////

RootFileWard170Page::RootFileWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "RootFileWard170Page";
	m_IsOK = FALSE;
	m_Path[0] = 0;
}

void RootFileWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_PATH), m_Path, sizeof(m_Path));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_PATH), m_Path);
	}

	m_IsOK = m_Path[0] != 0;
	PropertyPage::OnUpdate();
}

BOOL RootFileWard170Page::OnActivate()
{
	if (m_Path[0] == 0)
	{
		// Set the default root path
		sprintf(m_Path, "%s\\FTPRoot", m_pSetup->DestPath);
	}
	PropertyPage::OnActivate();
	return TRUE;
}

BOOL RootFileWard170Page::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (BUTTON(IDC_BROWSE))
	{
		OnBrowse();
		return TRUE;
	}

	return PropertyPage::OnMessage(uMsg, wParam, lParam);
}

void RootFileWard170Page::OnBrowse()
{
	LPMALLOC pMalloc;
  if (SHGetMalloc (&pMalloc)!= NOERROR)
		return;

  BROWSEINFO bInfo;
  LPITEMIDLIST pidl;
  ZeroMemory((PVOID) &bInfo,sizeof (BROWSEINFO));

  if (FALSE && m_Path[0])
  {
    OLECHAR       olePath[MAX_PATH];
    ULONG         chEaten;
    ULONG         dwAttributes;
    HRESULT       hr;
    LPSHELLFOLDER pDesktopFolder;
		if (SUCCEEDED(SHGetDesktopFolder(&pDesktopFolder)))
		{
      MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, m_Path, -1, olePath, MAX_PATH);
      hr = pDesktopFolder->ParseDisplayName(NULL,
				NULL, olePath, &chEaten, &pidl, &dwAttributes);
      if (FAILED(hr))
      {
        pMalloc ->Free (pidl);
        pMalloc ->Release ();
        return;
      }
      bInfo.pidlRoot = pidl;
    }
  }
  
	bInfo.hwndOwner = m_hWnd;
  bInfo.pszDisplayName = m_Path;
  bInfo.lpszTitle = "Select a location for the FTP Root file system";
  bInfo.ulFlags = BIF_RETURNFSANCESTORS|BIF_RETURNONLYFSDIRS ;
  bInfo.lpfn = NULL;  // address of callback function
  bInfo.lParam = (LPARAM)this;      // pass address of object to callback function

  if ((pidl = ::SHBrowseForFolder(&bInfo)) == NULL)
  {
    return;
  }

  if (::SHGetPathFromIDList(pidl,m_Path) == FALSE)
  {
    pMalloc ->Free(pidl);
    pMalloc ->Release();
    return;
  }

  pMalloc ->Free(pidl);
  pMalloc ->Release();

	OnUpdate(FALSE);
}

//////////////////////////////////////////////////////////////////////////
// UpgradePathWard170Page
//////////////////////////////////////////////////////////////////////////

UpgradePathWard170Page::UpgradePathWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "UpgradePathWard170Page";
	m_IsOK = FALSE;
	m_Path[0] = 0;
}

void UpgradePathWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_PATH), m_Path, sizeof(m_Path));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_PATH), m_Path);
	}

	// Check that we have the correct file here....

	m_DatName = "";
	char Path[MAX_PATH * 2];
	if (m_pOrigin)
	{
		SetupModeWard170Page *p = (SetupModeWard170Page *)m_pOrigin->GetPage("SetupModeWard170Page");
		if (p)
		{
			switch(p->m_Mode)
			{
				case SetupModeWard170Page::mode_upgr_pre170:
					m_DatName = "FtpDaemon.dat";
					break;
				case SetupModeWard170Page::mode_upgr_170:
					m_DatName = "WarUser.dat";
					break;
			}
		}
	}
	sprintf(Path, "%s\\%s", m_Path, m_DatName);

	m_IsOK = GetFileAttributes(Path) != 0xffffffff;
	PropertyPage::OnUpdate();
}

BOOL UpgradePathWard170Page::OnActivate()
{
	if (m_Path[0] == 0)
	{
		// Set the default root path
		sprintf(m_Path, "%s", m_pSetup->DestPath);
	}
	PropertyPage::OnActivate();
	return TRUE;
}

BOOL UpgradePathWard170Page::OnMessage(UINT uMsg,WPARAM wParam,LPARAM lParam)
{
	if (BUTTON(IDC_BROWSE))
	{
		OnBrowse();
		return TRUE;
	}

	return PropertyPage::OnMessage(uMsg, wParam, lParam);
}

void UpgradePathWard170Page::OnBrowse()
{
	LPMALLOC pMalloc;
  if (SHGetMalloc (&pMalloc)!= NOERROR)
		return;

  BROWSEINFO bInfo;
  LPITEMIDLIST pidl;
  ZeroMemory((PVOID) &bInfo,sizeof (BROWSEINFO));

  if (FALSE && m_Path[0])
  {
    OLECHAR       olePath[MAX_PATH];
    ULONG         chEaten;
    ULONG         dwAttributes;
    HRESULT       hr;
    LPSHELLFOLDER pDesktopFolder;
		if (SUCCEEDED(SHGetDesktopFolder(&pDesktopFolder)))
		{
      MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, m_Path, -1, olePath, MAX_PATH);
      hr = pDesktopFolder->ParseDisplayName(NULL,
				NULL, olePath, &chEaten, &pidl, &dwAttributes);
      if (FAILED(hr))
      {
        pMalloc ->Free (pidl);
        pMalloc ->Release ();
        return;
      }
      bInfo.pidlRoot = pidl;
    }
  }
  
	bInfo.hwndOwner = m_hWnd;
  bInfo.pszDisplayName = m_Path;
  bInfo.lpszTitle = "Select a previous installed server";
  bInfo.ulFlags = BIF_RETURNFSANCESTORS|BIF_RETURNONLYFSDIRS ;
  bInfo.lpfn = NULL;  // address of callback function
  bInfo.lParam = (LPARAM)this;      // pass address of object to callback function

  if ((pidl = ::SHBrowseForFolder(&bInfo)) == NULL)
  {
    return;
  }

  if (::SHGetPathFromIDList(pidl,m_Path) == FALSE)
  {
    pMalloc ->Free(pidl);
    pMalloc ->Release();
    return;
  }

  pMalloc ->Free(pidl);
  pMalloc ->Release();

	OnUpdate(FALSE);
}

//////////////////////////////////////////////////////////////////////////
// SystemNameWard170Page
//////////////////////////////////////////////////////////////////////////


SystemNameWard170Page::SystemNameWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "SystemNameWard170Page";
	m_IsOK = TRUE;
	m_SystemName[0] = 0;
}

void SystemNameWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_NAME), m_SystemName, sizeof(m_SystemName));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_NAME), m_SystemName);
	}

	m_IsOK = m_SystemName[0] != 0;
	PropertyPage::OnUpdate();
}

BOOL SystemNameWard170Page::OnActivate()
{
	if (m_SystemName[0] == 0)
	{
		// Set the default root path
		strcpy(m_SystemName, "Jgaa's Fan Club FTP service");
	}
	PropertyPage::OnActivate();
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// SecurityWard170Page
//////////////////////////////////////////////////////////////////////////

SecurityWard170Page::SecurityWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "SecurityWard170Page";
	m_IsOK = TRUE;

	m_bAllowAnon = TRUE;
	m_bListUpload = FALSE;
	m_bReadUpload = FALSE;
}

void SecurityWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		m_bAllowAnon = SendMessage(GetDlgItem(m_hWnd,IDC_ANON_ACCESS), BM_GETCHECK, 0, 0);
		m_bListUpload = SendMessage(GetDlgItem(m_hWnd,IDC_LIST_UPLOAD), BM_GETCHECK, 0, 0);
		m_bReadUpload = SendMessage(GetDlgItem(m_hWnd,IDC_READ_UPLOAD), BM_GETCHECK, 0, 0);
	}
	else
	{
		SendMessage(GetDlgItem(m_hWnd,IDC_ANON_ACCESS), BM_SETCHECK, m_bAllowAnon, 0);
		SendMessage(GetDlgItem(m_hWnd,IDC_LIST_UPLOAD), BM_SETCHECK, m_bListUpload, 0);
		SendMessage(GetDlgItem(m_hWnd,IDC_READ_UPLOAD), BM_SETCHECK, m_bReadUpload, 0);
	}

	PropertyPage::OnUpdate();
}


//////////////////////////////////////////////////////////////////////////
// PasswordWard170Page
//////////////////////////////////////////////////////////////////////////

PasswordWard170Page::PasswordWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "PasswordWard170Page";
	m_IsOK = FALSE;

	m_Passwd[0] = 0;
}

void PasswordWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_EDIT1), m_Passwd, sizeof(m_Passwd));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_EDIT1), m_Passwd);
	}

	char buf[sizeof(m_Passwd)];
	GetWindowText(GetDlgItem(m_hWnd, IDC_EDIT2), buf, sizeof(buf));

	m_IsOK = ((m_Passwd[0] != 0) && !strcmp(buf, m_Passwd));
	PropertyPage::OnUpdate();
}


//////////////////////////////////////////////////////////////////////////
// EmailWard170Page
//////////////////////////////////////////////////////////////////////////

EmailWard170Page::EmailWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "EmailWard170Page";
	m_IsOK = TRUE;

	m_Email[0] = 0;
}

void EmailWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_EDIT1), m_Email, sizeof(m_Email));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_EDIT1), m_Email);
	}
	PropertyPage::OnUpdate();
}

//////////////////////////////////////////////////////////////////////////
// NameTagWard170Page
//////////////////////////////////////////////////////////////////////////

NameTagWard170Page::NameTagWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "NameTagWard170Page";
	m_IsOK = TRUE;

	strcpy(m_Tag, "WARSVR");
}

void NameTagWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_EDIT1), m_Tag, sizeof(m_Tag));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_EDIT1), m_Tag);
	}

	m_IsOK = m_Tag[0] != 0;
	LPCSTR p = m_Tag;
	while(*p)
	{
		if (!isalpha(*p))
			m_IsOK = FALSE;
		++p;
	}

	if (!m_IsOK)
	{
		MessageBox(NULL, 
			"The name tag must consist of letters 'A' - 'Z' only, and can not be left blank.", 
			"Input error", MB_OK);
	}

	PropertyPage::OnUpdate();
}

/////////////////////////////////////////////////////////////////////////
// ServiceWard170Page
//////////////////////////////////////////////////////////////////////////

ServiceWard170Page::ServiceWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "ServiceWard170Page";
	m_IsOK = TRUE;

	m_bService = TRUE;
}

void ServiceWard170Page::OnUpdate(BOOL bSave)
{
	if (bSave)
	{
		m_bService = SendMessage(GetDlgItem(m_hWnd,IDC_CHECK1), BM_GETCHECK, 0, 0);
	}
	else
	{
		SendMessage(GetDlgItem(m_hWnd,IDC_CHECK1), BM_SETCHECK, m_bService, 0);
	}

	PropertyPage::OnUpdate();
}

/////////////////////////////////////////////////////////////////////////
// FTPPortWard170Page
//////////////////////////////////////////////////////////////////////////

FTPPortWard170Page::FTPPortWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "FTPPortWard170Page";
	m_IsOK = TRUE;
	m_Port = 21;
}

void FTPPortWard170Page::OnUpdate(BOOL bSave)
{
	char buf[16];

	sprintf(buf, "%d", m_Port);

	if (bSave)
	{
		GetWindowText(GetDlgItem(m_hWnd, IDC_PORT), buf, sizeof(buf));
	}
	else
	{
		SetWindowText(GetDlgItem(m_hWnd, IDC_PORT), buf);
	}

	m_Port = atoi(buf);

	m_IsOK = m_Port > 0;

	PropertyPage::OnUpdate();
}

/////////////////////////////////////////////////////////////////////////
// ServiceWard170Page
//////////////////////////////////////////////////////////////////////////

ReadyToGoWard170Page::ReadyToGoWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "ReadyToGoWard170Page";
	m_IsOK = TRUE;
}

/////////////////////////////////////////////////////////////////////////
// ProcessWard170Page
//////////////////////////////////////////////////////////////////////////

ProcessWard170Page::ProcessWard170Page(Setup *pSetup)
: PropertyPage(pSetup)
{
	m_Name = "ProcessWard170Page";
	m_IsOK = FALSE;
}

BOOL ProcessWard170Page::OnActivate()
{
	m_pOrigin->SetAsFirst("ProcessWard170Page");
	return PropertyPage::OnActivate();
}

#define SETUP_INI ".\\ward170setup.conf"
BOOL ProcessWard170Page::OnProcess()
{
	m_IsOK = TRUE;
	BOOL bDoService = TRUE;
	BOOL bDoTrayIcon = TRUE;
	BOOL bDoODBC = FALSE;
	DWORD Attr;
	LPCSTR TagName = "WARSVR";
	HINSTANCE h = NULL;
	BOOL (PASCAL *__SQLInstallDriverEx)(
		LPCSTR lpszDriver,
		LPCSTR lpszPathIn,
		LPSTR lpszPathOut,
		WORD cbPathOutMax,
		WORD * pcbPathOut,
		WORD fRequest,
		LPDWORD lpdwUsageCount) = NULL;

	BOOL (PASCAL *__SQLInstallTranslatorEx)(
		LPCSTR lpszTranslator,
		LPCSTR lpszPathIn,
		LPSTR lpszPathOut,
		WORD cbPathOutMax,
		WORD * pcbPathOut,
		WORD fRequest,
		LPDWORD lpdwUsageCount) = NULL;

	DeleteFile(SETUP_INI); // Delete any old setup files...
	SetupModeWard170Page *pSM = (SetupModeWard170Page *)m_pOrigin->GetPage("SetupModeWard170Page");

	if (!pSM->m_bDoClientInstall)
		EnableWindow(GetDlgItem(m_hWnd, IDC_REGFILES), FALSE);

	if (!pSM->m_bDoServerInstall)
	{
		EnableWindow(GetDlgItem(m_hWnd, IDC_REGSERVICE), FALSE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_SVCSETUP), FALSE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_STARTICON), FALSE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_STARTSVR), FALSE);
		EnableWindow(GetDlgItem(m_hWnd, IDC_REGODBC), FALSE);
		goto ClientInstall;
	}

	// See if odbc components was installed
	{
		InstallModePage *p = (InstallModePage *)m_pOrigin->GetPage("InstallModePage");
		if (p->m_Mode == InstallModePage::mode_custom)
		{
			InstallCustomPage *p = (InstallCustomPage *)m_pOrigin->GetPage("InstallCustomPage");
			for(int i =0; i < InstallCustomPage::NUM_MODULES; i++)
			{
				if (p->m_Data[i].m_pClass)
				{
					if (!stricmp(p->m_Data[i].m_pClass->m_Name, "DAO (ODBC and MS Jet Engine)"))
					{
						bDoODBC = TRUE;
						break;
					}
				}
			}
		}
		else if (p->m_Mode == InstallModePage::mode_normal)
			bDoODBC = TRUE;
	}

	if (bDoODBC)
	{
		h = LoadLibrary("odbccp32.dll");
		if (h)
		{
			__SQLInstallDriverEx = (BOOL (PASCAL *)(LPCSTR ,LPCSTR ,LPSTR ,WORD ,
				WORD * ,  WORD,LPDWORD)) ::GetProcAddress(h, "SQLInstallDriverEx");

			__SQLInstallTranslatorEx = (BOOL (PASCAL *)(LPCSTR ,LPCSTR ,LPSTR ,WORD ,
				WORD * ,  WORD,LPDWORD)) ::GetProcAddress(h, "SQLInstallTranslatorEx");
		}
	}

	if (!h || !__SQLInstallDriverEx)
		bDoODBC = FALSE;

	if (!bDoODBC)
		EnableWindow(GetDlgItem(m_hWnd, IDC_REGODBC), FALSE);
			
	
	// Write config file
	// Also dim out steps that is skipped
	if (!SetCurrentDirectory(m_pSetup->DestPath))
	{
		MessageBox(NULL, "Failed to change directory to install dir.", "War Setup error", MB_OK);
		PostQuitMessage(0);
		return TRUE;
	}
	Attr = GetFileAttributes(".\\WarTrayIcon.exe");
	if (Attr != 0xffffffff)
		bDoTrayIcon = TRUE;

	if (!bDoTrayIcon)
		EnableWindow(GetDlgItem(m_hWnd, IDC_STARTICON), FALSE);

	{
		ServiceWard170Page *p = (ServiceWard170Page *)m_pOrigin->GetPage("ServiceWard170Page");
		bDoService = p->m_bService;
		if (!bDoService)
		{
			SetWindowText(GetDlgItem(m_hWnd, IDC_REGSERVICE), "Unregister as system service");
		}
	}

	{
		RootFileWard170Page *p = (RootFileWard170Page *)m_pOrigin->GetPage("RootFileWard170Page");
		WritePrivateProfileString("setup", "RootFileSystem", p->m_Path, SETUP_INI);
	}

	{
		SystemNameWard170Page *p = (SystemNameWard170Page *)m_pOrigin->GetPage("SystemNameWard170Page");
		WritePrivateProfileString("setup", "SystemName", p->m_SystemName, SETUP_INI);
	}

	{
		SecurityWard170Page *p = (SecurityWard170Page *)m_pOrigin->GetPage("SecurityWard170Page");
		WritePrivateProfileInt("setup", "AllowAnon", p->m_bAllowAnon, SETUP_INI);
		WritePrivateProfileInt("setup", "ListUpload", p->m_bListUpload, SETUP_INI);
		WritePrivateProfileInt("setup", "ReadUpload", p->m_bReadUpload, SETUP_INI);
	}

	{
		FTPPortWard170Page *p = (FTPPortWard170Page *)m_pOrigin->GetPage("FTPPortWard170Page");
		WritePrivateProfileInt("setup", "FTPport", p->m_Port, SETUP_INI);
	}

	{
		PasswordWard170Page *p = (PasswordWard170Page *)m_pOrigin->GetPage("PasswordWard170Page");
		WritePrivateProfileString("setup", "Password", p->m_Passwd, SETUP_INI);
	}

	{
		EmailWard170Page *p = (EmailWard170Page *)m_pOrigin->GetPage("EmailWard170Page");
		WritePrivateProfileString("setup", "Email", p->m_Email, SETUP_INI);
	}

	{
		NameTagWard170Page *p = (NameTagWard170Page *)m_pOrigin->GetPage("NameTagWard170Page");
		WritePrivateProfileString("Options", "core_RESNAME", p->m_Tag, ".\\warsvr.conf");
		TagName = p->m_Tag;
	}

	if (pSM->m_Mode != SetupModeWard170Page::mode_new)
	{
		UpgradePathWard170Page *p = (UpgradePathWard170Page *)m_pOrigin->GetPage("UpgradePathWard170Page");
		WritePrivateProfileString("setup", "UpgradePath", p->m_Path, SETUP_INI);
		WritePrivateProfileString("setup", "UpgradeName", p->m_DatName, SETUP_INI);
	}

ClientInstall:
	// Register file types
	if (pSM->m_bDoClientInstall)
	{
		STARTUPINFO si;
		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		PROCESS_INFORMATION pi;
		memset(&pi, 0, sizeof(pi));

		CreateProcess(NULL, ".\\WDM.exe -install", NULL, NULL, FALSE, 
			0, NULL, NULL, &si, &pi);

		while(WaitForSingleObject(pi.hThread, 500) == WAIT_TIMEOUT)
			PumpMessages();

		SendMessage(GetDlgItem(m_hWnd, IDC_REGFILES), BM_SETCHECK, 1, 0);
	}

	if (!pSM->m_bDoServerInstall)
		goto Done;

	// Register ODBC drivrs
	if (bDoODBC)
	{
		char DummyPath[MAX_PATH];
		DWORD dwUsageCnt;
		WORD wDummyLen = sizeof(DummyPath);
		BOOL rrVal = __SQLInstallDriverEx(
			"Microsoft Access Driver (*.mdb)\0Driver=odbcjt32.dll\0Setup=odbcjt32.dll\0APILevel=1\0ConnectFunctions=YYN\0DriverODBCVer=02.50\0FileUsage=2\0FileExtns=*.mdb\0SQLLevel=0\0ConfigDSN=DSN=MS Access '97 Database|UID=Admin||\0\0",
			NULL,
			DummyPath,
			sizeof(DummyPath),
			&wDummyLen,
			ODBC_INSTALL_COMPLETE,
			&dwUsageCnt);

		rrVal = __SQLInstallDriverEx(
			"SQL Server\0Driver=sqlsrv32.dll\0Setup=sqlsrv32.dll\0SQLLevel=1\0FileUsage=0\0DriverODBCVer=02.50\0ConnectFunctions=YYY\0APILevel=2\0\0",
			NULL,
			DummyPath,
			sizeof(DummyPath),
			&wDummyLen,
			ODBC_INSTALL_COMPLETE,
			&dwUsageCnt);

		rrVal = __SQLInstallDriverEx(
			"Oracle SQL Server\0Driver=msorcl10.dll\0\0",
			NULL,
			DummyPath,
			sizeof(DummyPath),
			&wDummyLen,
			ODBC_INSTALL_COMPLETE,
			&dwUsageCnt);

		if (__SQLInstallTranslatorEx)
			rrVal = __SQLInstallTranslatorEx(
				"MS Code Page Translator\0Translator=MSCPXL32.DLL\0Setup=MSCPXL32.DLL\0\0",
				NULL,
				DummyPath,
				sizeof(DummyPath),
				&wDummyLen,
				ODBC_INSTALL_COMPLETE,
				&dwUsageCnt);

		SendMessage(GetDlgItem(m_hWnd, IDC_REGODBC), BM_SETCHECK, 1, 0);
	}

	// Register service
	if (bDoService)
	{
		STARTUPINFO si;
		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		PROCESS_INFORMATION pi;
		memset(&pi, 0, sizeof(pi));

		CreateProcess(NULL, ".\\war-ftpd.exe -setup -install", NULL, NULL, FALSE, 
			CREATE_NEW_PROCESS_GROUP, NULL, NULL, &si, &pi);

		while(WaitForSingleObject(pi.hThread, 500) == WAIT_TIMEOUT)
			PumpMessages();

		SendMessage(GetDlgItem(m_hWnd, IDC_REGSERVICE), BM_SETCHECK, 1, 0);
	}
	else
	{
		STARTUPINFO si;
		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		PROCESS_INFORMATION pi;
		memset(&pi, 0, sizeof(pi));

		CreateProcess(NULL, ".\\war-ftpd.exe -setup -uninstall", NULL, NULL, FALSE, 
			CREATE_NEW_PROCESS_GROUP, NULL, NULL, &si, &pi);

		while(WaitForSingleObject(pi.hThread, 500) == WAIT_TIMEOUT)
			PumpMessages();

		SendMessage(GetDlgItem(m_hWnd, IDC_REGSERVICE), BM_SETCHECK, 1, 0);
	}
	
	// Start server in setup mode
	{
		STARTUPINFO si;
		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		PROCESS_INFORMATION pi;
		memset(&pi, 0, sizeof(pi));

		CreateProcess(NULL, ".\\war-ftpd.exe -setup", NULL, NULL, FALSE, 
			CREATE_NEW_PROCESS_GROUP, NULL, NULL, &si, &pi);

		while(WaitForSingleObject(pi.hThread, 500) == WAIT_TIMEOUT)
			PumpMessages();

		SendMessage(GetDlgItem(m_hWnd, IDC_SVCSETUP), BM_SETCHECK, 1, 0);
	}

	// Start try icon
	if (bDoTrayIcon)
	{
		STARTUPINFO si;
		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		PROCESS_INFORMATION pi;
		memset(&pi, 0, sizeof(pi));

		CreateProcess(".\\WarTrayIcon.exe", NULL, NULL, NULL, FALSE, 
			CREATE_NEW_PROCESS_GROUP, NULL, NULL, &si, &pi);

		SendMessage(GetDlgItem(m_hWnd, IDC_STARTICON), BM_SETCHECK, 1, 0);
	}

	// Start the server
	if (bDoService &&	MyStartService(TagName))
	{
		;
	}
	else
	{
		STARTUPINFO si;
		memset(&si, 0, sizeof(si));
		si.cb = sizeof(si);
		PROCESS_INFORMATION pi;
		memset(&pi, 0, sizeof(pi));

		CreateProcess(NULL, ".\\war-ftpd.exe -notservice", NULL, NULL, FALSE, 
			CREATE_NEW_PROCESS_GROUP, NULL, NULL, &si, &pi);
	}

	SendMessage(GetDlgItem(m_hWnd, IDC_STARTSVR), BM_SETCHECK, 1, 0);

Done:
	
	DeleteFile(SETUP_INI);

	OnUpdate();
	return TRUE;
}

void WritePrivateProfileInt(LPCSTR Hdr, LPCSTR Tag, int val, LPCSTR File)
{
	char buf[32];
	sprintf(buf,"%d", val);
	WritePrivateProfileString(Hdr, Tag, buf, File);
}

BOOL MyStartService(LPCSTR Name)
{
	SERVICE_STATUS sStatus;
	// Open the Service Control Manager
  SC_HANDLE hSCM = ::OpenSCManager(NULL, // local machine
                                   NULL, // ServicesActive database
                                     SC_MANAGER_ALL_ACCESS); // full access
  if (!hSCM) return FALSE;

  BOOL bResult = FALSE;
  SC_HANDLE hService = ::OpenService(hSCM, Name, SERVICE_ALL_ACCESS);
  if (hService)
	{
		if (::StartService(hService,0,NULL)) 
		{
			if (QueryServiceStatus(hService, &sStatus))
			{
				DWORD cp;
				while(sStatus.dwCurrentState != SERVICE_RUNNING)
				{
					cp = sStatus.dwCheckPoint;
					Sleep(sStatus.dwWaitHint);
					if (!::QueryServiceStatus(hService, &sStatus))
						break;
					if ((cp >= sStatus.dwCheckPoint)
						&& (sStatus.dwCurrentState != SERVICE_START_PENDING))
						break;
				}
				bResult = (sStatus.dwCurrentState == SERVICE_RUNNING);
			}
		} 
		else 
		{
			;
		}
		::CloseServiceHandle(hService);
	}
	::CloseServiceHandle(hSCM);
	return bResult;
}


// Use all known methods to shut down all versions of War FTP Daemon
void ShutDownServers(Setup *pSetup)
{
	// Shut down the tray icon process
	HWND wTray;
	while((wTray = FindWindow("WarTrayIconClass", "WarTrayIcon")) != NULL)
	{
		PumpMessages();
		SendMessage(wTray, WM_CLOSE, 0, 0);
	}

	// Shut down pre 170 servers
	if ((pSetup->OsType == OS_NT4)
		|| (pSetup->OsType == OS_NT351))
	{
		// Try pre 170 service
		ControlService("WAR-FTPD", SERVICE_CONTROL_STOP, NULL);

		// Try 170 service
		// Don't enumerate alternate names. If such are in use, the operator
		// most likely know what he's doing.
		ControlService("WARSVR", SERVICE_CONTROL_STOP, NULL);
	}

	EnumWindows(EnumWindowsProc, NULL);
}

BOOL CALLBACK EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
#define MAX_WINS 32
	char Name[128];
	static HWND Processed[MAX_WINS];
	static int Cnt;

	if (GetClassName(hwnd, Name, sizeof(Name)))
	{
		if (!strcmp(Name, "#32770"))
		{
			GetWindowText(hwnd, Name, sizeof(Name));
			if (!strcmp(Name, "War FTP Daemon"))
			{
				for(int i = 0; i < MAX_WINS; i++)
				{
					if (Processed[i] == hwnd)
					{
						return FALSE;
					}
				}
				if (Cnt == MAX_WINS)
					return FALSE;
				Processed[Cnt++] = hwnd;
				for(i = 0; i < 10; i++)
				{
					SendMessage(hwnd, WM_CLOSE, 0, 0);
					Sleep(3000);
					PumpMessages();
				}
			}
		}
	}

	return TRUE;
}

BOOL ControlService(LPCSTR Name, DWORD Command, DWORD *pStatus)
{
	SERVICE_STATUS sStatus; 
	DWORD fdwAccess;

	/* The required service object access depends on the control. */ 
	
	switch (Command)
	{ 
		case SERVICE_CONTROL_STOP: 
			fdwAccess = SERVICE_STOP; 
			break; 
		
		case SERVICE_CONTROL_PAUSE: 
		case SERVICE_CONTROL_CONTINUE: 
			fdwAccess = SERVICE_PAUSE_CONTINUE; 
			break; 

		default: 
			Command = SERVICE_CONTROL_INTERROGATE;
			fdwAccess = SERVICE_INTERROGATE; 
			break;
  } 

	SC_HANDLE hSCM = ::OpenSCManager(NULL, // local machine
                                   NULL, // ServicesActive database
																	 SC_MANAGER_CONNECT);

	if (!hSCM) 
		return FALSE;


	BOOL bResult = FALSE;
  SC_HANDLE hService = ::OpenService(hSCM, Name, fdwAccess);
  if (hService)
	{
		if (::ControlService(hService,Command,&sStatus)) 
		{
			bResult = TRUE;
			if (pStatus)
				*pStatus = sStatus.dwCurrentState;
		} 
		else 
		{
			;
		}
		::CloseServiceHandle(hService);
	}
	::CloseServiceHandle(hSCM);
	return bResult;
} 


